# encoding: utf-8

import asyncio
from typing import Any, Awaitable, Callable, Optional

from aiotest.logger import logger


class LoadShapeManager:
    """负载形状管理器，负责管理负载测试的形状变化逻辑"""

    def __init__(self, load_shape: Any,
                 apply_load_callback: Callable[[int, float], Awaitable[Any]]):
        """
        初始化负载形状管理器

        参数:
            load_shape: 负载形状对象，需要实现 tick() 方法，返回 (user_count, rate) 元组或 None
            apply_load_callback: 应用负载的回调函数，接收 (user_count, rate) 参数
        """
        self.load_shape = load_shape
        self.apply_load_callback = apply_load_callback
        self._task: Optional[asyncio.Task] = None
        self._is_running = False

    async def start(self) -> None:
        """启动负载形状管理"""
        if self._is_running:
            logger.warning("负载形状管理器已经在运行")
            return

        if not self.load_shape:
            raise ValueError("负载形状是必需的但未提供")

        # 验证 load_shape 是否有 tick 方法
        if not hasattr(self.load_shape, "tick") or not callable(
                self.load_shape.tick):
            raise ValueError("负载形状必须有一个 tick() 方法")

        self._is_running = True
        self._task = asyncio.create_task(
            self._run_shape_test(),
            name="load_shape_manager")
        logger.info("负载形状管理器已启动")

    async def stop(self) -> None:
        """停止负载形状管理"""
        if not self._is_running:
            return

        self._is_running = False

        if self._task and not self._task.done():
            self._task.cancel()
            try:
                await self._task
            except asyncio.CancelledError:
                logger.info("负载形状管理器已被取消")

        self._task = None
        logger.info("负载形状管理器已停止")

    async def _handle_error(self, error: Exception,
                            consecutive_errors: int) -> int:
        """
        处理错误并返回更新后的连续错误计数

        参数:
            error: 发生的错误
            consecutive_errors: 当前连续错误计数

        返回:
            更新后的连续错误计数
        """
        logger.error("形状测试错误: %s", str(error))
        consecutive_errors += 1
        if consecutive_errors > 3:
            logger.error("连续错误过多，停止形状测试")
        await asyncio.sleep(1)
        return consecutive_errors

    async def _run_shape_test(self) -> None:
        """形状测试主逻辑"""
        logger.info("形状测试正在启动")
        shape_last = None  # 上次配置
        consecutive_errors = 0  # 连续错误计数

        try:
            while self._is_running:
                try:
                    # 获取下一个形状配置
                    shape_new = self.load_shape.tick()

                    # 检查是否应该停止
                    if shape_new is None:
                        logger.info("形状测试已停止（收到None）")
                        break

                    # 检查是否与上次相同
                    if shape_last == shape_new:
                        await asyncio.sleep(1)
                        continue

                    # 解析新配置
                    try:
                        user_count, rate = shape_new
                        logger.info(
                            "负载更新: %d 个用户，速率: %.2f/s", user_count, rate)

                        # 应用新配置 - 调用回调函数
                        await self.apply_load_callback(user_count, rate)
                        shape_last = shape_new
                        consecutive_errors = 0  # 重置错误计数

                    except (ValueError, TypeError) as e:
                        consecutive_errors = await self._handle_error(e, consecutive_errors)
                        if consecutive_errors > 3:
                            break

                except Exception as e:
                    consecutive_errors = await self._handle_error(e, consecutive_errors)
                    if consecutive_errors > 3:
                        break

        except asyncio.CancelledError:
            logger.info("形状测试已被取消")

    @property
    def is_running(self) -> bool:
        """检查是否正在运行"""
        return self._is_running

    @property
    def task(self) -> Optional[asyncio.Task]:
        """获取后台任务"""
        return self._task


class StandaloneLoadShapeRunner:
    """独立的负载形状运行器，用于不需要继承BaseRunner的场景"""

    def __init__(self, load_shape: Any,
                 apply_load_callback: Callable[[int, float], Awaitable[Any]]):
        """
        初始化独立负载形状运行器

        参数:
            load_shape: 负载形状对象，需要实现 tick() 方法
            apply_load_callback: 应用负载的回调函数，接收 (user_count, rate) 参数
        """
        self.load_shape_manager = LoadShapeManager(
            load_shape, apply_load_callback)
        self._quit_flag = False

    async def start(self) -> None:
        """启动形状测试"""
        if self._quit_flag:
            raise RuntimeError("退出后无法启动")

        await self.load_shape_manager.start()

    async def quit(self) -> None:
        """退出形状测试"""
        self._quit_flag = True
        await self.load_shape_manager.stop()

    @property
    def is_quit(self) -> bool:
        """检查是否已退出"""
        return self._quit_flag

    @property
    def is_running(self) -> bool:
        """检查是否正在运行"""
        return self.load_shape_manager.is_running



# encoding: utf-8
"""
HTTP客户端模块，提供异步HTTP请求功能

封装了基于aiohttp的异步HTTP请求功能，支持连接池管理、自动重试、请求/响应拦截器等特性。
"""

import asyncio
import json
import re
import time
import traceback
import uuid
from functools import lru_cache
from typing import Any, Callable, Dict, List, Optional, Union

import aiohttp
import backoff

from aiotest.events import request_metrics
from aiotest.logger import logger
from aiotest.metrics import RequestMetrics

# 连接池配置
CONNECTOR_SETTINGS = {
    'limit': 1000,  # 默认最大连接数（支持高并发场景）
    'limit_per_host': 100,  # 默认每个主机最大连接数（适合单主机压测）
    'force_close': False,  # 默认是否强制关闭空闲连接
    'enable_cleanup_closed': True,  # 默认是否清理已关闭的连接
}


def configure_connector(limit=None, limit_per_host=None,
                        force_close=None, enable_cleanup_closed=None):
    """动态配置连接池参数"""
    for key, value in {
        'limit': limit,
        'limit_per_host': limit_per_host,
        'force_close': force_close,
        'enable_cleanup_closed': enable_cleanup_closed
    }.items():
        if value is not None:
            CONNECTOR_SETTINGS[key] = value

    logger.info("当前连接池配置: %s", CONNECTOR_SETTINGS)


class HTTPClient:
    """
    异步HTTP客户端类，封装了基于aiohttp的异步HTTP请求功能。

    功能：
    - 提供异步HTTP请求功能（GET/POST/PUT/DELETE等）。
    - 支持连接池管理，优化高并发场景下的性能。
    - 自动重试机制，处理网络波动或临时故障。
    - 请求/响应拦截器，支持自定义逻辑扩展。
    - 敏感信息过滤，避免日志泄露敏感数据。
    - 统一的错误处理，简化异常管理。

    示例：
        >>> async with HTTPClient(base_url="https://api.example.com") as client:
        ...     response = await client.get("/users")
        ...     print(await response.json())

    参数：
        base_url (str): 基础URL，所有请求将基于此URL构建。默认为空字符串。
        default_headers (Optional[Dict[str, str]]): 默认请求头。如果未提供，将使用内置默认值。
        timeout (int): 请求超时时间（秒）。默认为30秒。
        max_retries (int): 最大重试次数。默认为3次。
        verify_ssl (bool): 是否验证SSL证书。默认为True。

    异常：
        ValueError: 如果参数值无效（如超时时间为负数）。
    """

    def __init__(
        self,
        base_url: str = "",
        default_headers: Optional[Dict[str, str]] = None,
        timeout: int = 30,
        max_retries: int = 3,
        verify_ssl: bool = True,
    ):
        """
        初始化HTTP客户端。

        参数：
            base_url (str): 基础URL，所有请求将基于此URL构建。默认为空字符串。
            default_headers (Optional[Dict[str, str]]): 默认请求头。如果未提供，将使用内置默认值。
            timeout (int): 请求超时时间（秒）。默认为30秒。
            max_retries (int): 最大重试次数。默认为3次。
            verify_ssl (bool): 是否验证SSL证书。默认为True。

        异常：
            ValueError: 如果参数值无效（如超时时间为负数）。
        """

        self.base_url = base_url
        self.default_headers = default_headers or {}
        self.timeout = aiohttp.ClientTimeout(total=timeout)
        self.max_retries = max_retries
        self.verify_ssl = verify_ssl
        self._session = None
        self._connector = None
        # 预编译敏感字段正则
        self._sensitive_field_pattern = re.compile(
            r'(password|secret|token|credit_card|cvv|api[_-]key|auth)', re.IGNORECASE
        )
        self._sensitive_str_pattern = re.compile(
            r'(password|token|api[_-]key|auth)=[^&]+', re.IGNORECASE
        )

        # 设置默认请求头
        self.default_headers.update({
            'Accept': 'application/json',
            'Accept-Encoding': 'gzip, deflate',
            'Accept-Language': 'zh-CN,zh;q=0.9',
        })

    async def __aenter__(self):
        """
        异步上下文管理器入口，用于初始化HTTP会话和连接池。

        功能：
        - 初始化TCP连接池。
        - 创建ClientSession实例。

        返回：
            HTTPClient: 当前实例，支持链式调用。

        异常：
            ClientError: 如果会话初始化失败。
        """
        self._connector = aiohttp.TCPConnector(
            limit=CONNECTOR_SETTINGS['limit'],
            limit_per_host=CONNECTOR_SETTINGS['limit_per_host'],
            force_close=CONNECTOR_SETTINGS['force_close'],
            enable_cleanup_closed=CONNECTOR_SETTINGS['enable_cleanup_closed'],
            verify_ssl=self.verify_ssl
        )
        self._session = aiohttp.ClientSession(
            base_url=self.base_url,
            headers=self.default_headers,
            timeout=self.timeout,
            connector=self._connector,
            json_serialize=json.dumps  # 使用标准json序列化
        )
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """关闭HTTP会话和连接池，清理资源"""
        close_timeout = 5.0  # 秒

        if self._session:
            try:
                await asyncio.wait_for(self._session.close(), timeout=close_timeout)
            except asyncio.TimeoutError:
                logger.warning("会话关闭超时")

    async def close(self):
        """手动关闭HTTP客户端并清理资源"""
        await self.__aexit__(None, None, None)

    async def _request_with_retry(
        self,
        method: str,
        endpoint: str,
        **kwargs
    ) -> aiohttp.ClientResponse:
        """
        带重试的实际请求执行方法。

        参数：
            method (str): HTTP方法（如GET、POST等）。
            endpoint (str): API端点路径。
            **kwargs (Any): 其他请求参数（如headers、json等）。

        返回：
            aiohttp.ClientResponse: 响应对象。
        """

        # 当max_retries=0时完全跳过重试逻辑
        if self.max_retries == 0:
            return await self._session.request(method, endpoint, **kwargs)

        @backoff.on_exception(
            backoff.expo,
            (aiohttp.ClientError, asyncio.TimeoutError, ConnectionError, OSError),
            max_tries=self.max_retries,
            logger=logger,
            giveup=lambda e: isinstance(
                e, aiohttp.ClientResponseError) and e.status in [
                400, 401, 403, 404]
        )
        async def _do_request():
            """执行实际请求"""
            return await self._session.request(method, endpoint, **kwargs)

        return await _do_request()

    def request(
        self,
        method: str,
        endpoint: str,
        name: Optional[str] = None,
        **kwargs
    ) -> "ResponseContextManager":
        """
        发送HTTP请求

        参数：
            method: HTTP方法 (GET, POST, PUT, DELETE等)
            endpoint: API端点 (不包含基础URL)
            name: 请求名称
            **kwargs: 其他aiohttp请求参数

        返回：
            ResponseContextManager: 响应上下文管理器对象
        """

        # 使用高精度时间戳 + 随机数确保唯一性
        request_id = f"req-{int(time.monotonic() * 1000000)}-{uuid.uuid4().hex[:8]}"
        # 保留原始 endpoint 用于实际请求
        request_endpoint = endpoint
        # 处理日志和标准化
        log_endpoint = endpoint.lstrip('/')
        if not log_endpoint:
            log_endpoint = "unknown"
        normalized_endpoint = self._normalize_endpoint(log_endpoint)

        # 准备请求参数
        kwargs.setdefault('headers', {}).update(self.default_headers)
        kwargs['ssl'] = self.verify_ssl

        # 结构化日志
        log = RequestMetrics(
            request_id=request_id,
            method=method,
            endpoint=name if name else normalized_endpoint,
            extra={
                "headers": self._sanitize_headers(kwargs.get('headers', {})),
                "params": kwargs.get('params', {}),
                "body": self._sanitize_body(kwargs.get('data', kwargs.get('json', None)))
            }
        )

        self._log_request(log, "start")

        start_time = time.monotonic()

        # 返回上下文管理器
        return ResponseContextManager(
            request_coroutine=self._request_with_retry(
                method, request_endpoint, **kwargs),
            log=log,
            start_time=start_time,
            metrics_callback=self._record_request_metrics,
            log_request=self._log_request
        )

    def get(self, endpoint: str,
            params: Optional[Dict[str, Any]] = None, **kwargs) -> "ResponseContextManager":
        """GET请求方法"""
        return self.request('GET', endpoint, params=params, **kwargs)

    def post(self, endpoint: str, data: Optional[Union[Dict[str, Any], str]] = None,
             **kwargs) -> "ResponseContextManager":
        """POST请求方法"""
        return self.request('POST', endpoint, data=data, **kwargs)

    def put(self, endpoint: str, data: Optional[Union[Dict[str, Any], str]] = None,
            **kwargs) -> "ResponseContextManager":
        """PUT请求方法"""
        return self.request('PUT', endpoint, data=data, **kwargs)

    def delete(self, endpoint: str, **kwargs) -> "ResponseContextManager":
        """DELETE请求方法"""
        return self.request('DELETE', endpoint, **kwargs)

    @lru_cache(maxsize=100)
    def _normalize_endpoint(self, endpoint: str) -> str:
        """
        将动态路径转换为固定模式

        参数：
        - endpoint: 原始API端点

        返回：
        - str: 标准化后的端点（自动转换 /users/123 为 /users/{id}）
        """
        return re.sub(r'\/\d+(/|$)', r'/{id}\1', endpoint)

    async def _record_request_metrics(
        self,
        request_id: str,
        method: str,
        endpoint: str,
        status_code: int,
        duration: float,
        response_size: int,
        error: Optional[Dict[str, Any]] = None,
        assertion_result: str = "unknown"
    ):
        """
        记录接口请求性能数据

        职责：
        - 收集接口请求的原始性能数据
        - 通过 request_events 事件上传数据
        - 不关心数据存储和上报方式
        """
        try:
            # 直接使用 RequestMetrics 类实例，保持数据结构一致性
            request_metrics_data = RequestMetrics(
                request_id=request_id,
                method=method,
                endpoint=endpoint,
                status_code=status_code,
                duration=duration,
                response_size=response_size,
                error=error,
                assertion_result=assertion_result
            )

            # 通过事件系统上传请求数据
            await request_metrics.fire(metrics=request_metrics_data)

            logger.debug(
                "记录请求指标: %s %s - %s (%.3fs)",
                method, endpoint, status_code, duration
            )

        except Exception as e:
            logger.warning("记录请求指标失败: %s", e)

    def _log_request(self, log: RequestMetrics, status: str):
        """
        统一记录请求日志

        参数：
        - log: 请求指标对象
        - status: 请求状态（start/complete/failed）
        """
        log_data = {
            "method": log.method,
            "endpoint": log.endpoint,
            "status": status,
        }
        log_prefix = f"request_{status} {log.method} {log.endpoint}"

        if status == "start":
            self._log_request_start(log, log_data, log_prefix)
        elif status == "complete":
            self._log_request_complete(log, log_data, log_prefix)
        elif status == "failed":
            self._log_request_failed(log, log_data, log_prefix)

    def _log_request_start(self, log: RequestMetrics, log_data: Dict[str, Any], log_prefix: str):
        """记录请求开始日志"""
        log_data["request_start"] = log.extra if isinstance(
            log.extra, dict) else {}
        logger.debug(log_prefix, extra=log_data)

    def _log_request_complete(self, log: RequestMetrics, log_data: Dict[str, Any], log_prefix: str):
        """记录请求完成日志"""
        # 过滤掉headers，避免日志过大
        if log.extra and isinstance(log.extra, dict):
            filtered_extra = {k: v for k, v in log.extra.items() if k != 'headers'}
        else:
            filtered_extra = {}
        log_data["response_data"] = filtered_extra
        logger.debug(log_prefix, extra=log_data)

    def _log_request_failed(self, log: RequestMetrics, log_data: Dict[str, Any], log_prefix: str):
        """记录请求失败日志"""
        if hasattr(log, 'error') and log.error is not None:
            log_data["error"] = log.error
            # 如果有响应数据,也记录到日志中
            if log.error.get("response_data") is not None:
                response_data = log.error["response_data"]
                # 限制响应数据长度
                if isinstance(response_data, (dict, str)):
                    response_str = json.dumps(
                        response_data, ensure_ascii=False) if isinstance(
                        response_data, dict) else str(response_data)
                    if len(response_str) > 500:
                        response_str = response_str[:500] + "..."
                    log_data["response_data"] = response_str
                else:
                    log_data["response_data"] = str(response_data)[:500]
        logger.error(log_prefix, extra=log_data)

    @staticmethod
    def _sanitize_headers(headers: Dict[str, str]) -> Dict[str, str]:
        """
        清理敏感信息的请求头

        参数：
        - headers: 原始请求头

        返回：
        - Dict[str, str]: 清理后的请求头
        """
        sensitive_keys = {'authorization', 'api-key', 'token', "cookie"}
        return {
            k: '*****' if any(s in k.lower() for s in sensitive_keys) else v
            for k, v in headers.items()
        }

    def _sanitize_body(self, body: Any) -> Any:
        """
        清理敏感信息的请求体

        参数：
        - body: 原始请求体

        返回：
        - Any: 清理后的请求体
        """
        if isinstance(body, dict):
            return {
                k: '*****' if self._sensitive_field_pattern.search(k) else v
                for k, v in body.items()
            }
        if isinstance(body, str):
            return self._sensitive_str_pattern.sub(r'\1=*****', body)
        return body


class ResponseContextManager:
    """
    响应上下文管理器类

    功能：
    - 封装响应处理逻辑
    - 支持异步上下文管理
    - 提供统一的响应处理方法
    """

    def __init__(
        self,
        request_coroutine,
        log: RequestMetrics,
        start_time: float,
        metrics_callback: Callable,
        log_request: Callable,
    ):
        """
        初始化响应上下文管理器

        参数：
        - request_coroutine: 请求协程
        - log: 请求指标对象
        - start_time: 请求开始时间
        - metrics_callback: 指标回调函数
        - log_request: 请求日志回调函数
        """
        self._request_coroutine = request_coroutine
        self.log = log
        self.start_time = start_time
        self._metrics_callback = metrics_callback
        self._log_request = log_request
        self._response = None
        self._processed_data = None
        self._error_handled = False

    async def __aenter__(self) -> Dict[str, Any]:
        """进入上下文，返回处理后的响应数据"""
        try:
            self._response = await self._request_coroutine
            self._processed_data = await self._process_response(self._response)
            duration = time.monotonic() - self.start_time

            # 更新日志
            self.log.status_code = self._response.status
            self.log.duration = duration
            self.log.response_size = self._estimate_size(
                self._processed_data.get('data'))
            # 确保 extra 不为 None
            if self.log.extra is None:
                self.log.extra = {}
            self.log.extra["response_data"] = self._processed_data.get('data')
        except Exception as e:
            # 处理请求错误（如超时、连接错误等）
            self.log.assertion_result = "fail"
            await self._handle_error(e)
            # 重新抛出异常，让测试用例捕获
            raise

        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """
        异步上下文管理器出口，用于清理资源。

        功能：关闭ClientSession和连接池，确保资源释放，避免内存泄漏。

        异常：asyncio.TimeoutError - 如果关闭操作超时。
        """
        if self._response:
            self._response.close()

        if exc_type:
            # 断言失败时，设置断言结果为fail
            self.log.assertion_result = "fail"
            await self._handle_error(exc_type, exc_val, exc_tb)
            return False  # 返回False，给users 顺序执行任务时，一个任务失败，后续任务不执行

        # 断言成功时，设置断言结果为pass
        self.log.assertion_result = "pass"
        # 记录成功指标
        logger.debug(
            "调用指标回调函数，方法: %s, 端点: %s",
            self.log.method, self.log.endpoint
        )
        await self._metrics_callback(
            request_id=self.log.request_id,
            method=self.log.method,
            endpoint=self.log.endpoint,
            status_code=self.log.status_code,
            duration=self.log.duration,
            response_size=self.log.response_size,
            assertion_result=self.log.assertion_result
        )
        # 记录成功日志
        self._log_request(self.log, "complete")
        return True

    @property
    def status(self) -> int:
        """提供 status 属性，保持和aiohttp一致的操作"""
        return self._processed_data['status']

    @property
    def headers(self) -> Dict[str, str]:
        """提供 headers 属性，保持和aiohttp一致的操作"""
        return self._processed_data['headers']

    async def text(self) -> str:
        """提供 text() 方法,保持和aiohttp一致的操作"""
        return self._processed_data['data']

    async def json(self) -> dict:
        """提供 json() 方法,保持和aiohttp一致的操作"""
        return self._processed_data['data']

    async def _process_response(
            self, response: aiohttp.ClientResponse) -> Dict[str, Any]:
        """处理响应数据"""
        try:
            raw_bytes = await response.read()
            data = self._parse_response_data(raw_bytes)

            # 统一处理 headers 和 cookies 转换
            headers_dict = self._safe_get_headers(response)
            cookies_dict = self._safe_get_cookies(response)

            return {
                'status': response.status,
                'headers': headers_dict,
                'data': data,
                'cookies': cookies_dict,
            }
        except Exception as e:
            logger.error("处理响应数据失败: %s", e)
            # 简化异常处理，使用空字典作为默认值
            return {
                'status': getattr(response, 'status', 500),
                'headers': self._safe_get_headers(response) if response else {},
                'data': None,
                'cookies': self._safe_get_cookies(response) if response else {},
                'error': str(e)
            }

    def _parse_response_data(self, raw_bytes: bytes) -> Any:
        """解析响应数据"""
        try:
            return json.loads(raw_bytes.decode('utf-8', errors='replace'))
        except Exception:
            # JSON 解析失败，尝试作为文本读取
            return raw_bytes.decode('utf-8', errors='replace')

    @staticmethod
    def _safe_get_headers(response) -> Dict[str, str]:
        """安全获取headers字典"""
        try:
            return dict(response.headers)
        except (TypeError, AttributeError):
            try:
                return dict(response.headers.items())
            except (TypeError, AttributeError):
                return {'Content-Type': 'application/json'}

    @staticmethod
    def _safe_get_cookies(response) -> Dict[str, Any]:
        """安全获取cookies字典"""
        try:
            if hasattr(response, 'cookies') and response.cookies:
                return dict(response.cookies)
        except (TypeError, AttributeError):
            pass
        return {}

    async def _handle_error(self, exc_type, exc_val=None, exc_tb=None):
        """处理请求错误"""
        # 检查是否已经处理过错误，避免重复记录
        if self._error_handled:
            return

        # 标记错误已处理
        self._error_handled = True

        # 简化参数处理逻辑
        if exc_val is None:
            exc_val, exc_type, exc_tb = exc_type, type(
                exc_type), exc_type.__traceback__

        duration = time.monotonic() - self.start_time

        # 断言失败时，设置断言结果为fail
        self.log.assertion_result = "fail"

        # 更新日志
        self.log.duration = duration
        self.log.endpoint = self.log.endpoint or "unknown"

        # 获取简化的堆栈信息
        simplified_stack = self._get_simplified_stack(exc_tb)

        # 获取错误消息，如果为空则提供默认值
        error_message = str(exc_val).strip() or f"{exc_type.__name__} occurred"

        # 获取接口返回信息并附加到错误消息
        error_message = self._enrich_error_message(error_message)

        # 安全获取响应数据
        processed_data = getattr(self, '_processed_data', {})
        response_data = processed_data.get("data") if isinstance(processed_data, dict) else None
        
        self.log.error = {
            "message": error_message,
            "exc_type": exc_type.__name__,
            "stack": simplified_stack,
            "response_data": response_data
        }

        # 记录失败指标
        await self._metrics_callback(
            request_id=self.log.request_id,
            method=self.log.method,
            endpoint=self.log.endpoint,
            status_code=self.log.status_code,
            duration=self.log.duration,
            response_size=0,
            error=self.log.error,
            assertion_result=self.log.assertion_result
        )

        # 记录失败日志
        self._log_request(self.log, "failed")

    def _get_simplified_stack(self, exc_tb) -> List[str]:
        """获取简化的堆栈信息"""
        simplified_stack = []
        if exc_tb:
            stack = traceback.extract_tb(exc_tb)
            simplified_stack = [
                f"{frame.name}() at {frame.filename}:{frame.lineno}"
                for frame in stack if "aiohttp/internal" not in frame.filename
            ][:3]
        return simplified_stack

    def _enrich_error_message(self, error_message: str) -> str:
        """丰富错误消息，添加响应数据"""
        processed_data = getattr(self, '_processed_data', {})
        response_data = processed_data.get("data") if isinstance(processed_data, dict) else None

        if response_data is not None:
            # 将响应数据转换为字符串（限制长度避免过长）
            if isinstance(response_data, dict):
                response_str = json.dumps(response_data, ensure_ascii=False)
            elif isinstance(response_data, bytes):
                response_str = response_data.decode('utf-8', errors='ignore')
            else:
                response_str = str(response_data)

            # 限制响应数据长度，避免错误消息过长
            if len(response_str) > 500:
                response_str = f"{response_str[:500]}..."

            error_message = f"{error_message} | Response: {response_str}"

        return error_message

    @staticmethod
    def _estimate_size(data: Any) -> int:
        """
        估算数据大小

        参数：
        - data: 数据对象

        返回：
        - int: 估算的字节大小
        """
        if data is None:
            return 0
        return len(str(data).encode('utf-8'))



# encoding: utf-8

from abc import ABCMeta, abstractmethod
from timeit import default_timer
from typing import Optional, Tuple


class LoadUserShape(metaclass=ABCMeta):
    """
    负载测试形状控制的抽象基类。

    功能：
        - 定义控制负载测试形状的基本接口。
        - 子类必须实现 `tick` 方法来定义具体的负载变化规律。

    属性：
        start_time (float): 测试开始时间，用于计算运行时长。
        paused_time (float): 暂停开始的时间。
        is_paused (bool): 是否处于暂停状态。
        pause_time (float): 累计的暂停时间。

    方法：
        reset_time(): 重置开始时间。
        get_run_time(): 获取运行时长。
        pause(): 暂停计时。
        resume(): 恢复计时。
        tick(): 抽象方法，子类必须实现。

    示例：
        class MyLoadShape(LoadUserShape):
            def tick(self):
                return (100, 5.0)  # 目标用户数100，速率5.0用户/秒
    """

    def __init__(self):
        self.start_time = default_timer()
        self.paused_time = 0.0
        self.is_paused = False
        self.pause_time = 0.0

    def reset_time(self) -> None:
        """
        重置开始时间，将计时器重置为当前时间。

        功能：
            - 用于重新开始计时，通常在测试重启时调用。

        说明：
            - 调用此方法后，`get_run_time` 将重新从0开始计算运行时长。
    """
        self.start_time = default_timer()
        self.paused_time = 0.0
        self.is_paused = False
        self.pause_time = 0.0

    def get_run_time(self) -> float:
        """
        获取从开始时间到当前时间的运行时长，减去暂停的时间。

        功能：
            - 计算从 `start_time` 到当前时间的运行时长，减去累计的暂停时间。

        返回：
            float: 运行时长（秒）。

        示例：
            >>> shape = LoadUserShape()
            >>> shape.get_run_time()
            10.5  # 运行了10.5秒
    """
        if self.is_paused:
            current_time = self.paused_time
        else:
            current_time = default_timer()
        return current_time - self.start_time - self.pause_time

    def pause(self) -> None:
        """
        暂停计时，记录暂停开始的时间。

        功能：
            - 当测试暂停时，调用此方法暂停计时。
    """
        if not self.is_paused:
            self.paused_time = default_timer()
            self.is_paused = True

    def resume(self) -> None:
        """
        恢复计时，计算并累加暂停的时间。

        功能：
            - 当测试恢复时，调用此方法恢复计时，并累加暂停的时间。
    """
        if self.is_paused:
            self.pause_time += default_timer() - self.paused_time
            self.is_paused = False

    @abstractmethod
    def tick(self) -> Optional[Tuple[int, float]]:
        """
        获取当前时刻的负载控制参数，用于控制负载用户的数量变化。

        返回：
            Optional[Tuple[int, float]]:
                - 当返回一个元组 (user_count, rate) 时：
                    * user_count (int): 目标总用户数。
                    * rate (float): 每秒启动或停止的用户数。
                - 当返回 None 时：停止当前的负载测试。

        说明：
            - 这是一个抽象方法，子类必须实现此方法来定义具体的负载变化策略。
            - 通常用于动态调整负载测试的用户数量和速率。
        """
        return None



# encoding: utf-8

import asyncio
from typing import Any, Callable, Dict, List, Optional, Type

from aiotest.logger import logger
from aiotest.users import User


class UserManager:
    """用户管理器，负责用户创建、启动、停止和权重分配"""

    def __init__(self, user_types: List[Type['User']], config: Dict[str, Any]):
        self.user_types: List[Type['User']] = user_types
        self.config: Dict[str, Any] = config
        self.active_users: List['User'] = []

    def _calculate_weighted_counts(
        self,
        user_types: List[Type['User']],
        target_count: int,
        current_counts: Optional[Dict[Type['User'], int]] = None
    ) -> Dict[Type['User'], int]:
        """
        通用的权重分配算法。

        参数：
            user_types (List[Type['User']]): 用户类型列表
            target_count (int): 目标分配总数
            current_counts (Optional[Dict[Type['User'], int]]): 当前各类型的用户数量（用于限制）

        返回：
            Dict[Type['User'], int]: 各用户类型到分配数量的字典

        异常：
            ValueError: 权重配置无效时抛出
        """
        if not user_types:
            return {}

        # 验证并获取权重
        weights: List[int] = []
        for user_type in user_types:
            weight = getattr(user_type, 'weight', 1)
            if not isinstance(weight, int) or weight < 1:
                raise ValueError(
                    f"用户类 {user_type.__name__} 必须有一个 'weight' 属性（整数 ≥ 1）")
            weights.append(weight)

        # 单一用户类型直接返回
        if len(user_types) == 1:
            user_type = user_types[0]
            if current_counts is not None:
                count = min(target_count, current_counts.get(user_type, 0))
            else:
                count = target_count
            return {user_type: count}

        # 计算权重总和
        total_weight = sum(weights)
        if total_weight == 0:
            return {}

        # 按权重比例计算分配数量
        counts: List[int] = []
        for user_type, weight in zip(user_types, weights):
            ratio = weight / total_weight
            count = int(round(ratio * target_count))

            # 如果有当前数量限制，确保不超过
            if current_counts is not None:
                count = min(count, current_counts.get(user_type, 0))
            count = max(count, 0)
            counts.append(count)

        # 处理舍入误差
        while sum(counts) != target_count:
            if sum(counts) < target_count:
                # 优先增加当前数量最多的类型
                if current_counts is not None:
                    idx = max(range(len(counts)),
                              key=lambda i: current_counts[user_types[i]])
                else:
                    idx = counts.index(min(counts))

                if current_counts is None or counts[idx] < current_counts[user_types[idx]]:
                    counts[idx] += 1
                else:
                    break
            else:
                # 优先减少数量最多的类型
                idx = max(range(len(counts)), key=lambda i: counts[i])
                if counts[idx] > 0:
                    counts[idx] -= 1
                else:
                    break

        return {user_type: counts[i] for i, user_type in enumerate(user_types)}

    def distribute_users_by_weight(
            self, user_count: int) -> List[Type['User']]:
        """
        基于权重的用户分配算法。

        参数：
            user_count (int): 目标用户总数。

        返回：
            List[Type['User']]: 按权重分配后的用户类列表。

        异常：
            ValueError: 权重配置无效时抛出。

        示例：
            # 假设有两个用户类：MyUser1(weight=2), MyUser2(weight=3)，目标100个用户
            # 分配结果：MyUser1 -> 40个 (2/5 * 100)，MyUser2 -> 60个 (3/5 * 100)
            user_manager = UserManager([MyUser1, MyUser2], {})
            result = user_manager.distribute_users_by_weight(100)
            # 返回: [MyUser1, MyUser1, ..., MyUser2, MyUser2, ...] (40个MyUser1, 60个MyUser2)
        """
        if not self.user_types:
            return []

        # 使用通用权重分配算法
        counts = self._calculate_weighted_counts(self.user_types, user_count)

        # 生成用户类列表
        result = []
        for user_type, count in counts.items():
            result.extend([user_type for _ in range(count)])
        return result

    async def manage_users(
        self,
        user_count: int,
        rate: float,
        action: str
    ) -> None:
        """
        用户管理的核心方法（支持平滑启停）

        参数:
            user_count: 目标用户数
            rate: 操作速率(个/秒)
            action: 操作类型('start'或'stop')

        异常:
            ValueError: 非法操作类型或参数值时抛出
        """
        # 验证参数
        if not isinstance(user_count, int) or user_count < 0:
            raise ValueError("user_count 必须是一个非负整数")
        if not isinstance(rate, (int, float)) or rate < 0:
            raise ValueError("rate 必须是一个非负数")

        if action == "start":
            await self._start_users(user_count, rate)
        elif action == "stop":
            await self._stop_users(user_count, rate)
        else:
            raise ValueError("action 必须是 'start' 或 'stop'")

    async def _start_users(self, user_count: int, rate: float):
        """启动指定数量的用户（按权重分配）"""
        user_classes = self.distribute_users_by_weight(user_count)
        await self._batch_execute(user_classes, self._create_and_start_user, rate)

    async def _stop_users(self, user_count: int, rate: float):
        """停止指定数量的用户（按权重比例）"""
        target_users = await self._select_users_to_stop(user_count)
        await self._batch_execute(target_users, self._stop_user, rate)

    async def _batch_execute(
            self, items: List, operation: Callable, rate: float):
        """
        批量操作执行器，支持速率控制

        参数：
            items (List): 需要处理的项目列表
            operation (Callable): 执行的异步操作函数
            rate (float): 操作速率（个/秒），0表示无限制

        示例：
            # rate=5.0时，每0.2秒处理一个项目
            # 10个项目预计耗时：1.8秒（最后一个不延迟）
            await _batch_execute([user1, user2], start_user, 5.0)
            # 执行顺序：start_user(user1) -> sleep(0.2) -> start_user(user2)

        逻辑说明：
            1. 计算操作间隔：delay = 1.0 / rate
               - rate=5.0 -> delay=0.2秒
               - rate=0 -> delay=0（无限制）
            2. 顺序处理每个项目：
               - 执行操作并统计成功数
               - 操作间按速率延迟（最后一个操作不延迟）
               - 捕获单个操作异常，不影响其他操作
            3. 记录处理结果（成功时记录debug日志）
        """
        success_count = 0
        delay = 1.0 / rate if rate > 0 else 0

        for i, item in enumerate(items):
            try:
                await operation(item)
                success_count += 1

                # 速率控制：最后一个操作后不需要延迟
                if i < len(items) - 1 and delay > 0:
                    await asyncio.sleep(delay)
            except Exception as e:
                logger.warning("操作失败，项目 %d: %s", i, e)
                continue

        if success_count > 0:
            logger.debug(
                "成功处理 %d/%d 个项目", success_count, len(items))

    async def _create_and_start_user(self, user_class: Type['User']):
        """创建并启动单个用户"""
        new_user: 'User' = user_class()
        # 设置 host（如果用户类需要）
        # 兼容字典和 Namespace 对象
        if hasattr(self.config, 'host'):
            # 处理 Namespace 对象
            host = getattr(self.config, 'host', None)
        else:
            # 处理字典对象
            host = self.config.get('host', None)
        if host and hasattr(new_user, 'host'):
            new_user.host = host

        new_user.start_tasks()
        self.active_users.append(new_user)

    async def _stop_user(self, user: 'User'):
        """终止单个用户"""
        if user.tasks is not None and not user.tasks.done():
            await user.stop_tasks()

    async def _select_users_to_stop(self, user_count: int):
        """
        选择要停止的用户（按权重比例）

        参数：
            user_count (int): 需要停止的用户数量

        返回：
            List['User']: 需要停止的用户实例列表

        示例：
            # 当前状态：MyUser1(weight=2): 40个, MyUser2(weight=3): 60个, 总计100个
            # 目标停止30个用户
            result = await _select_users_to_stop(30)
            # 返回：包含30个用户的列表，其中MyUser1约12个，MyUser2约18个
            # 停止后剩余：MyUser1约28个，MyUser2约42个，保持2:3的权重比例

        逻辑说明：
            1. 获取所有活跃用户（tasks未完成的）
            2. 如果停止数量≥活跃数量，返回所有活跃用户
            3. 按用户类型分组，统计各类型数量
            4. 单一用户类型时，按启动顺序停止
            5. 多用户类型：
               - 计算当前各类型数量
               - 计算停止后的目标总用户数
               - 调用权重分配算法计算各类型应停止数量
               - 按启动顺序选择要停止的用户
        """
        active_users = [
            user for user in self.active_users if user.tasks is not None and not user.tasks.done()]

        if user_count >= len(active_users):
            return active_users

        # 按用户类型分组
        users_by_type: Dict[Type['User'], List['User']] = {}
        for user in active_users:
            user_type = type(user)
            if user_type not in users_by_type:
                users_by_type[user_type] = []
            users_by_type[user_type].append(user)

        # 如果只有一种用户类型，按启动顺序停止
        if len(users_by_type) == 1:
            return active_users[:user_count]

        # 计算当前各类型数量
        current_counts = {
            user_type: len(users) for user_type,
            users in users_by_type.items()}
        user_types = list(current_counts.keys())

        # 直接使用权重分配算法计算各类型应停止的数量
        stop_counts = self._calculate_weighted_counts(
            user_types, user_count, current_counts)

        # 按启动顺序选择要停止的用户
        users_to_stop = []
        for user_type, stop_count in stop_counts.items():
            if stop_count > 0:
                users_to_stop.extend(users_by_type[user_type][:stop_count])

        return users_to_stop

    @property
    def active_user_count(self) -> int:
        """返回当前活跃用户数量"""
        return sum(
            1 for user in self.active_users if user.tasks is not None and not user.tasks.done())

    def cleanup_inactive_users(self):
        """清理非活跃用户实例（已完成的任务）"""
        self.active_users = [
            user for user in self.active_users if user.tasks is not None and not user.tasks.done()]

    async def stop_all_users(self):
        """停止所有用户"""
        for user in self.active_users:
            if not user.tasks.done():
                await user.stop_tasks()
        self.active_users.clear()

    async def pause_all_users(self):
        """暂停所有用户"""
        for user in self.active_users:
            if not user.tasks.done():
                await user.pause_tasks()

    async def resume_all_users(self):
        """恢复所有用户"""
        for user in self.active_users:
            if not user.tasks.done():
                await user.resume_tasks()



# encoding: utf-8

import asyncio
import sys
import traceback
from typing import Any, Callable, Dict, List, Tuple

from aiotest.logger import logger

# ============================================================================
# 核心事件系统类
# ============================================================================


class EventHook:
    """
    异步事件钩子，支持优先级和线程安全。

    功能：
        - 基于优先级的事件处理器执行。
        - 线程安全的处理器添加/移除操作。
        - 支持批量触发事件。
        - 支持装饰器式注册。

    使用场景：
        - 动态注册事件处理器。
        - 并发执行事件处理器。
        - 处理高优先级任务。
    """

    def __init__(self):
        self._handlers: List[Tuple[int, Callable[..., Any]]] = []
        self._lock = asyncio.Lock()
        # 待注册的装饰器处理器
        self._pending_handlers: List[Tuple[Callable[..., Any], int]] = []

    def handler(self, priority: int = 0):
        """
        事件处理器装饰器 - 用于声明式注册事件处理器

        参数：
            priority (int): 优先级，数值越大优先级越高，默认为 0

        使用示例：
            from aiotest import test_start

            @test_start.handler(priority=0)
            async def my_test_start_handler(**kwargs):
                print("测试开始")

        返回：
            装饰器函数
        """
        def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
            self._pending_handlers.append((func, priority))
            return func
        return decorator

    async def add_handler(
        self,
        handler: Callable[..., Any],
        priority: int = 0
    ) -> None:
        """
        添加事件处理器。

        参数：
            handler (Callable[..., Any]): 事件处理器函数。
            priority (int, 可选): 处理器的优先级，数值越高越先执行。默认为 0。
        """
        if not callable(handler):
            raise TypeError("处理器必须是可调用的")
        handler_name = getattr(handler, "__name__", repr(handler))
        async with self._lock:
            # 使用对象标识进行去重，避免同名方法被误判
            for _, existing_handler in self._handlers:
                if existing_handler is handler:
                    logger.warning("处理器 %s 已存在", handler_name)
                    return  # 如果处理器对象已存在，直接返回不添加
            # 使用优先级作为排序键，避免直接比较函数对象
            self._handlers.append((-priority, handler))
            self._handlers.sort(key=lambda x: x[0])
            logger.info(
                "已添加处理器 %s，优先级: %d", handler_name, priority)

    async def register_pending_handlers(self):
        """
        注册所有通过装饰器定义的待注册处理器

        通常在 init_events 触发时调用此方法，
        将所有使用 @event_hook.handler() 装饰器定义的处理器注册到此事件钩子中。
        """
        handler_count = len(self._pending_handlers)
        for handler, priority in self._pending_handlers:
            await self.add_handler(handler, priority=priority)
        self._pending_handlers.clear()
        return handler_count

    async def remove_handler(self, handler: Callable[..., Any]) -> None:
        """
        移除事件处理器。

        参数：
            handler (Callable[..., Any]): 要移除的事件处理器函数。
        """
        async with self._lock:
            self._handlers = [h for h in self._handlers if h[1] != handler]

    async def fire(self, **kwargs: Any) -> None:
        """
        触发事件。

        参数：
            **kwargs (Any): 传递给事件处理器的参数。
        """
        handlers = [h[1] for h in self._handlers]  # 提取排序后的处理器
        tasks = [self._safe_execute(h, **kwargs) for h in handlers]  # 创建任务
        await asyncio.gather(*tasks, return_exceptions=True)  # 并发执行

    async def fire_batch(self, events: List[Dict[str, Any]]) -> None:
        """
        批量触发事件。

        参数：
            events (List[Dict[str, Any]]): 事件字典列表。

        异常：
            RuntimeError: 如果任务数量超过限制（1000）。
        """
        if len(events) * len(self._handlers) > 1000:
            raise RuntimeError("批量任务数量过多")
        handlers = [h[1] for h in self._handlers]  # 提取排序后的处理器
        # 创建任务：生成器
        tasks = (
            self._safe_execute(handler, **event)
            for event in events
            for handler in handlers
        )
        await asyncio.gather(*tasks, return_exceptions=True)

    async def _safe_execute(
        self,
        handler: Callable[..., Any],
        **kwargs: Any
    ) -> None:
        """
        安全执行事件处理器，支持超时和错误处理。

        参数：
            handler (Callable[..., Any]): 事件处理器函数。
            **kwargs (Any): 传递给处理器的参数。
        """
        try:
            # 检查处理器类型
            if asyncio.iscoroutinefunction(handler):
                # 协程函数，直接调用并等待
                await asyncio.wait_for(handler(**kwargs), timeout=5.0)
            else:
                # 调用处理器
                result = handler(**kwargs)
                # 如果返回协程对象，等待它
                if asyncio.iscoroutine(result):
                    await asyncio.wait_for(result, timeout=5.0)
                else:
                    # 同步函数，在执行器中运行
                    loop = asyncio.get_event_loop()
                    await loop.run_in_executor(None, lambda: handler(**kwargs))
        except asyncio.TimeoutError:
            logger.warning(
                "处理器超时: %s",
                getattr(
                    handler,
                    '__name__',
                    repr(handler)))
        except Exception as e:
            logger.error(
                "事件处理器 %s 发生错误:\n错误: %s\n堆栈: %s",
                getattr(handler, '__name__', repr(handler)),
                e,
                ''.join(traceback.format_tb(sys.exc_info()[2]))
            )


class Events:
    """
    集中式事件系统管理。

    功能：
    - 动态事件注册。
    - 自动创建事件钩子。
    - 预定义测试生命周期的核心事件。
    - 自动注册装饰器定义的处理器。
    """

    def __init__(self):
        self._events: Dict[str, EventHook] = {}

    def register(self, name: str) -> EventHook:
        """
        注册新事件类型。

        参数：
            name (str): 事件类型名称。

        返回：
            EventHook: 注册事件的事件钩子实例。
        """
        if name not in self._events:
            self._events[name] = EventHook()
        return self._events[name]

    async def register_all_pending_handlers(self):
        """
        注册所有事件钩子的待注册处理器

        通常在 init_events 触发时调用此方法，
        将所有使用装饰器定义的处理器注册到对应的事件钩子中。
        """
        total_count = 0
        for event_hook in self._events.values():
            count = await event_hook.register_pending_handlers()
            total_count += count
        logger.info("已注册 %d 个事件处理器（通过装饰器）", total_count)

    def __getattr__(self, name: str) -> EventHook:
        """
        获取事件钩子，如果不存在则自动注册。

        参数：
            name (str): 事件钩子名称。

        返回：
            EventHook: 事件钩子实例。
        """
        if name not in self._events:
            self.register(name)
        return self._events[name]

    def __dir__(self):
        return list(self._events.keys())


# 全局事件系统实例 - 用于动态事件注册
events = Events()

# 预定义核心事件 - 直接导出使用
# 测试生命周期事件
init_events = events.register("init_events")  # 初始化事件
test_start = events.register("test_start")  # 测试开始事件
test_stop = events.register("test_stop")    # 测试停止事件
test_quit = events.register("test_quit")    # 测试退出事件
startup_completed = events.register("startup_completed")  # 启动完成事件

# 性能指标事件
# 请求指标事件(local节点：性能数据报告给prometheus，worker节点: 性能数据推送给redis)
request_metrics = events.register("request_metrics")
worker_request_metrics = events.register(
    "worker_request_metrics")  # master节点接收worker请求指标事件

# ============================================================================
# 导出所有事件以支持直接导入
# ============================================================================

__all__ = [
    # 全局事件系统
    'events',
    'EventHook',

    # 测试生命周期事件
    'init_events',
    'test_start',
    'test_stop',
    'test_quit',
    'startup_completed',

    # 性能指标事件
    'request_metrics',
    'worker_request_metrics',
]



# encoding: utf-8

import sys
import asyncio

# 检查Python版本
if sys.version_info < (3, 9):
    print("错误：AioTest 需要 Python 3.9 或更高版本。")
    print(f"您正在使用 Python {sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro}")
    sys.exit(1)

from aiotest.main import main as async_main


def main():
    """控制台脚本入口函数（同步）"""
    
    # 设置 WindowsSelectorEventLoopPolicy 以解决兼容性问题
    if sys.platform == "win32":
        asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
    
    # 导入并执行异步 main 函数（从 main.py 导入）
    asyncio.run(async_main())


if __name__ == "__main__":
    main()


# encoding: utf-8

"""
AioTest 事件系统测试模块

测试 EventHook 和 Events 类的功能，包括：
- 事件处理器注册和移除
- 装饰器方式注册
- 优先级执行顺序
- 同步/异步处理器
- 事件触发和批量触发
- 异常处理
- 去重机制
"""

import asyncio

import pytest

from aiotest.events import EventHook, Events

# ============================================================================
# EventHook 基础功能测试
# ============================================================================


class TestEventHookBasic:
    """测试 EventHook 基础功能"""

    @pytest.mark.asyncio
    async def test_add_handler(self):
        """测试添加事件处理器"""
        event_hook = EventHook()
        called = False

        async def handler(**kwargs):
            nonlocal called
            called = True

        await event_hook.add_handler(handler)
        await event_hook.fire()

        assert called, "处理器应该被调用"

    @pytest.mark.asyncio
    async def test_add_handler_with_priority(self):
        """测试带优先级的处理器执行顺序"""
        event_hook = EventHook()
        results = []

        async def handler_low(**kwargs):
            results.append("low")

        async def handler_high(**kwargs):
            results.append("high")

        async def handler_medium(**kwargs):
            results.append("medium")

        # 按不同优先级添加
        await event_hook.add_handler(handler_low, priority=0)
        await event_hook.add_handler(handler_high, priority=10)
        await event_hook.add_handler(handler_medium, priority=5)

        await event_hook.fire()

        # 高优先级应该先执行
        assert results == [
            "high", "medium", "low"], f"执行顺序应该是 high -> medium -> low, 实际: {results}"

    @pytest.mark.asyncio
    async def test_remove_handler(self):
        """测试移除事件处理器"""
        event_hook = EventHook()
        called = False

        async def handler(**kwargs):
            nonlocal called
            called = True

        await event_hook.add_handler(handler)
        await event_hook.remove_handler(handler)
        await event_hook.fire()

        assert not called, "处理器应该被移除"

    @pytest.mark.asyncio
    async def test_duplicate_handler_prevention(self):
        """测试重复处理器注册的去重机制"""
        event_hook = EventHook()
        call_count = 0

        async def handler(**kwargs):
            nonlocal call_count
            call_count += 1

        # 尝试添加同一个处理器两次
        await event_hook.add_handler(handler, priority=0)
        await event_hook.add_handler(handler, priority=0)

        await event_hook.fire()

        assert call_count == 1, f"处理器应该只被调用一次，实际调用了 {call_count} 次"

    @pytest.mark.asyncio
    async def test_non_callable_handler_error(self):
        """测试传入非可调用对象时抛出 TypeError"""
        event_hook = EventHook()

        with pytest.raises(TypeError, match="处理器必须是可调用的"):
            await event_hook.add_handler("not a function")


# ============================================================================
# EventHook 装饰器功能测试
# ============================================================================

class TestEventHookDecorator:
    """测试 EventHook 装饰器功能"""

    @pytest.mark.asyncio
    async def test_handler_decorator(self):
        """测试使用装饰器注册处理器"""
        event_hook = EventHook()
        called = False

        @event_hook.handler(priority=0)
        async def my_handler(**kwargs):
            nonlocal called
            called = True

        # 注册待注册的处理器
        await event_hook.register_pending_handlers()
        await event_hook.fire()

        assert called, "通过装饰器注册的处理器应该被调用"

    @pytest.mark.asyncio
    async def test_handler_decorator_with_priority(self):
        """测试装饰器的优先级"""
        event_hook = EventHook()
        results = []

        @event_hook.handler(priority=10)
        async def handler_high(**kwargs):
            results.append("high")

        @event_hook.handler(priority=0)
        async def handler_low(**kwargs):
            results.append("low")

        await event_hook.register_pending_handlers()
        await event_hook.fire()

        assert results == [
            "high", "low"], f"执行顺序应该是 high -> low, 实际: {results}"

    @pytest.mark.asyncio
    async def test_decorator_and_manual_registration(self):
        """测试装饰器和手动注册混合使用"""
        event_hook = EventHook()
        results = []

        # 装饰器注册
        @event_hook.handler(priority=10)
        async def handler_decorator(**kwargs):
            results.append("decorator")

        # 手动注册
        async def handler_manual(**kwargs):
            results.append("manual")

        await event_hook.add_handler(handler_manual, priority=5)
        await event_hook.register_pending_handlers()
        await event_hook.fire()

        assert results == [
            "decorator", "manual"], f"执行顺序应该是 decorator -> manual, 实际: {results}"


# ============================================================================
# EventHook 同步/异步处理器测试
# ============================================================================

class TestEventHookHandlerTypes:
    """测试同步和异步事件处理器"""

    @pytest.mark.asyncio
    async def test_async_handler(self):
        """测试异步处理器"""
        event_hook = EventHook()
        called = False

        async def async_handler(**kwargs):
            nonlocal called
            called = True

        await event_hook.add_handler(async_handler)
        await event_hook.fire()

        assert called, "异步处理器应该被调用"

    @pytest.mark.asyncio
    async def test_sync_handler(self):
        """测试同步处理器"""
        event_hook = EventHook()
        called = False

        def sync_handler(**kwargs):
            nonlocal called
            called = True

        await event_hook.add_handler(sync_handler)
        await event_hook.fire()

        assert called, "同步处理器应该被调用"

    @pytest.mark.asyncio
    async def test_mixed_handlers(self):
        """测试混合同步和异步处理器"""
        event_hook = EventHook()
        results = []

        async def async_handler(**kwargs):
            results.append("async")

        def sync_handler(**kwargs):
            results.append("sync")

        await event_hook.add_handler(async_handler)
        await event_hook.add_handler(sync_handler)
        await event_hook.fire()

        assert "async" in results and "sync" in results, "同步和异步处理器都应该被调用"

    @pytest.mark.asyncio
    async def test_handler_returning_coroutine(self):
        """测试返回协程对象的同步函数"""
        event_hook = EventHook()
        called = False

        async def inner_async():
            nonlocal called
            called = True

        def sync_handler(**kwargs):
            return inner_async()

        await event_hook.add_handler(sync_handler)
        await event_hook.fire()

        assert called, "返回协程的同步函数应该被执行"


# ============================================================================
# EventHook 事件参数测试
# ============================================================================

class TestEventHookParameters:
    """测试事件处理器参数传递"""

    @pytest.mark.asyncio
    async def test_handler_parameters(self):
        """测试处理器接收参数"""
        event_hook = EventHook()
        received_params = {}

        async def handler(**kwargs):
            received_params.update(kwargs)

        test_params = {"key1": "value1", "key2": 42}
        await event_hook.add_handler(handler)
        await event_hook.fire(**test_params)

        assert received_params == test_params, f"处理器应该接收正确的参数, 期望: {test_params}, 实际: {received_params}"

    @pytest.mark.asyncio
    async def test_multiple_handlers_with_parameters(self):
        """测试多个处理器共享事件参数"""
        event_hook = EventHook()
        results = []

        async def handler1(**kwargs):
            results.append(("handler1", kwargs.get("value")))

        async def handler2(**kwargs):
            results.append(("handler2", kwargs.get("value")))

        await event_hook.add_handler(handler1)
        await event_hook.add_handler(handler2)
        await event_hook.fire(value=42)

        assert results == [("handler1", 42), ("handler2", 42)
                           ], f"所有处理器应该接收相同的参数, 实际: {results}"


# ============================================================================
# EventHook 异常处理测试
# ============================================================================

class TestEventHookExceptions:
    """测试异常处理机制"""

    @pytest.mark.asyncio
    async def test_handler_exception_handling(self):
        """测试处理器异常不影响其他处理器"""
        event_hook = EventHook()
        results = []

        async def handler_with_error(**kwargs):
            results.append("error_handler")
            raise ValueError("Test error")

        async def handler_normal(**kwargs):
            results.append("normal_handler")

        await event_hook.add_handler(handler_with_error)
        await event_hook.add_handler(handler_normal)

        # fire 应该捕获异常，继续执行其他处理器
        await event_hook.fire()

        assert "error_handler" in results, "抛异常的处理器应该被调用"
        assert "normal_handler" in results, "正常的处理器应该继续执行"

    @pytest.mark.asyncio
    async def test_handler_timeout(self):
        """测试处理器超时处理"""
        event_hook = EventHook()
        timed_out = False

        async def slow_handler(**kwargs):
            await asyncio.sleep(10)  # 超过默认 5 秒超时

        async def normal_handler(**kwargs):
            nonlocal timed_out
            timed_out = True

        await event_hook.add_handler(slow_handler)
        await event_hook.add_handler(normal_handler)
        await event_hook.fire()

        assert timed_out, "正常处理器应该执行完毕"


# ============================================================================
# EventHook 批量触发测试
# ============================================================================

class TestEventHookBatchFire:
    """测试批量触发事件"""

    @pytest.mark.asyncio
    async def test_fire_batch(self):
        """测试批量触发事件"""
        event_hook = EventHook()
        call_count = 0

        async def handler(**kwargs):
            nonlocal call_count
            call_count += 1

        await event_hook.add_handler(handler)

        events = [{"id": i, "value": i * 10} for i in range(5)]
        await event_hook.fire_batch(events)

        assert call_count == 5, f"应该触发 5 次，实际触发 {call_count} 次"

    @pytest.mark.asyncio
    async def test_fire_batch_with_parameters(self):
        """测试批量触发时参数正确传递"""
        event_hook = EventHook()
        received_values = []

        async def handler(**kwargs):
            received_values.append(kwargs.get("id"))

        await event_hook.add_handler(handler)

        events = [{"id": i} for i in range(3)]
        await event_hook.fire_batch(events)

        assert received_values == [
            0, 1, 2], f"参数应该按顺序传递, 期望: [0, 1, 2], 实际: {received_values}"

    @pytest.mark.asyncio
    async def test_fire_batch_limit(self):
        """测试批量触发任务数量限制"""
        event_hook = EventHook()
        call_count = 0

        async def handler(**kwargs):
            nonlocal call_count
            call_count += 1

        await event_hook.add_handler(handler)

        # 创建超过 1000 个任务
        events = [{"id": i} for i in range(1010)]

        with pytest.raises(RuntimeError, match="批量任务数量过多"):
            await event_hook.fire_batch(events)


# ============================================================================
# Events 类测试
# ============================================================================

class TestEventsClass:
    """测试 Events 类功能"""

    def test_register_event(self):
        """测试注册新事件"""
        events = Events()
        event_hook = events.register("custom_event")

        assert isinstance(event_hook, EventHook), "应该返回 EventHook 实例"
        assert event_hook in events._events.values(), "事件应该被注册"

    def test_getattr_auto_register(self):
        """测试通过 __getattr__ 自动注册事件"""
        events = Events()
        event_hook = events.custom_event

        assert isinstance(event_hook, EventHook), "应该自动注册并返回 EventHook 实例"
        assert "custom_event" in events._events, "事件应该被自动注册"

    def test_dir_method(self):
        """测试 __dir__ 方法返回事件名称"""
        events = Events()
        events.register("event1")
        events.register("event2")

        dir_result = dir(events)

        assert "event1" in dir_result, "event1 应该在 dir 结果中"
        assert "event2" in dir_result, "event2 应该在 dir 结果中"

    @pytest.mark.asyncio
    async def test_register_all_pending_handlers(self):
        """测试注册所有待注册处理器"""
        events = Events()
        called = False

        @events.custom_event.handler(priority=0)
        async def my_handler(**kwargs):
            nonlocal called
            called = True

        await events.register_all_pending_handlers()
        await events.custom_event.fire()

        assert called, "待注册处理器应该被注册并调用"


# ============================================================================
# 并发安全测试
# ============================================================================

class TestEventHookConcurrency:
    """测试并发安全"""

    @pytest.mark.asyncio
    async def test_concurrent_add_handlers(self):
        """测试并发添加处理器"""
        event_hook = EventHook()

        async def add_handlers():
            for i in range(10):
                async def handler(**kwargs):
                    pass
                await event_hook.add_handler(handler, priority=i)

        # 并发添加多个处理器
        await asyncio.gather(add_handlers(), add_handlers(), add_handlers())

        # 应该有 30 个处理器（不去重的情况下）
        # 由于每个 handler 都是独立的函数，不会触发去重
        assert len(
            event_hook._handlers) == 30, f"应该有 30 个处理器, 实际: {len(event_hook._handlers)}"

    @pytest.mark.asyncio
    async def test_concurrent_add_and_remove(self):
        """测试在异步上下文中并发添加和移除处理器"""
        event_hook = EventHook()
        call_count = 0

        async def handler(**kwargs):
            nonlocal call_count
            call_count += 1

        async def add_and_remove():
            await event_hook.add_handler(handler)
            await asyncio.sleep(0.01)
            await event_hook.remove_handler(handler)

        # 并发添加和移除
        await asyncio.gather(add_and_remove(), add_and_remove(), add_and_remove())

        # 触发事件，检查是否正常运行
        await event_hook.fire()

        # 由于有并发，可能添加成功也可能失败，只要不崩溃即可
        assert isinstance(call_count, int)


# ============================================================================
# 集成测试
# ============================================================================

class TestEventsIntegration:
    """测试完整的事件系统集成"""

    @pytest.mark.asyncio
    async def test_full_lifecycle(self):
        """测试完整的事件生命周期"""
        events = Events()
        lifecycle = []

        # 定义生命周期事件
        @events.init.handler(priority=0)
        async def on_init(**kwargs):
            lifecycle.append("init")

        @events.start.handler(priority=0)
        async def on_start(**kwargs):
            lifecycle.append("start")

        @events.stop.handler(priority=0)
        async def on_stop(**kwargs):
            lifecycle.append("stop")

        # 注册所有待注册处理器
        await events.register_all_pending_handlers()

        # 模拟生命周期
        await events.init.fire()
        await events.start.fire()
        await events.stop.fire()

        assert lifecycle == [
            "init", "start", "stop"], f"生命周期应该是 init -> start -> stop, 实际: {lifecycle}"

    @pytest.mark.asyncio
    async def test_priority_based_execution(self):
        """测试基于优先级的完整执行顺序"""
        events = Events()
        results = []

        # 定义命名函数（避免每次创建新的 lambda 对象）
        async def handler_low(**kwargs):
            results.append(3)

        async def handler_high(**kwargs):
            results.append(1)

        async def handler_medium(**kwargs):
            results.append(2)

        # 添加不同优先级的处理器
        await events.test_event.add_handler(handler_low, priority=0)
        await events.test_event.add_handler(handler_high, priority=10)
        await events.test_event.add_handler(handler_medium, priority=5)

        await events.test_event.fire()

        assert results == [1, 2, 3], f"优先级顺序应该是 10 -> 5 -> 0, 实际: {results}"



# encoding: utf-8

import asyncio
from typing import TYPE_CHECKING, Any, Dict, List, Optional, Type

import psutil
from redis.asyncio import Redis

from aiotest.events import startup_completed, test_start, worker_request_metrics
from aiotest.exception import InvalidRateError, InvalidUserCountError
from aiotest.load_shape_manager import LoadShapeManager
from aiotest.logger import logger
from aiotest.metrics import RequestMetrics
from aiotest.state_manager import RunnerState, StateManager
from aiotest.task_manager import TaskManager
from aiotest.user_manager import UserManager

# ============================================================================
# 运行器节点类型常量
# ============================================================================
NODE_TYPE_LOCAL = "local"
NODE_TYPE_MASTER = "master"
NODE_TYPE_WORKER = "worker"
if TYPE_CHECKING:
    from aiotest.users import User


def validate_load_params(user_count: int, rate: float) -> None:
    """
    验证负载测试参数

    参数:
        user_count: 用户数量，必须为正整数
        rate: 速率，必须大于0且不超过用户数量

    异常:
        InvalidUserCountError: 当user_count不是正整数时抛出
        InvalidRateError: 当rate不在有效范围内时抛出
    """
    if not isinstance(user_count, int) or user_count <= 0:
        raise InvalidUserCountError(
            f"用户数量必须是正整数，当前值: {user_count}")

    if rate <= 0 or rate > user_count:
        raise InvalidRateError(
            f"速率必须在 0 和 {user_count} 之间，当前值: {rate}")

    # 速率过高警告
    if rate > 100:
        logger.warning(
            "高生成率（>100 用户/秒）可能导致性能问题")


def validate_params(func):
    """
    参数验证装饰器，验证用户数量和速率参数

    Args:
        func: 被装饰的函数

    Returns:
        包装后的函数
    """

    async def wrapper(self, user_count: int, rate: float, *args, **kwargs):
        validate_load_params(user_count, rate)
        return await func(self, user_count, rate, *args, **kwargs)
    return wrapper


class EventHandlerRegistry:
    """事件处理器注册中心"""

    def __init__(self):
        self._registered_handlers = set()

    async def register_handlers(self, runner):
        """根据运行器类型注册相应的事件处理器"""
        runner_key = id(runner)

        if runner_key in self._registered_handlers:
            return  # 避免重复注册

        # 注册通用事件处理器（仅非Worker节点）
        if not hasattr(runner, 'node') or runner.node != NODE_TYPE_WORKER:
            await startup_completed.add_handler(on_startup_completed)

        # 仅Master节点需要注册worker相关事件处理器，用于收集Worker上报的指标
        if hasattr(runner, 'node') and runner.node == NODE_TYPE_MASTER:
            await worker_request_metrics.add_handler(on_worker_request_metrics)

        self._registered_handlers.add(runner_key)
        logger.debug(
            "已为 %s 运行器 %s 注册事件处理器",
            getattr(runner, 'node', 'unknown'),
            runner_key)


class RunnerFactory:
    """运行器工厂，负责创建和初始化不同类型的运行器"""

    @staticmethod
    async def create(
        runner_type: str,
        user_types: List[Type['User']],
        load_shape: Any,
        config: Dict[str, Any],
        redis_client: Optional[Redis] = None
    ) -> Any:
        """
        创建指定类型的运行器实例

        参数：
            runner_type: 运行器类型 ("local", "master", "worker")
            user_types: 用户类列表
            load_shape: 负载形状类
            config: 配置参数
            redis_client: Redis客户端（分布式模式需要）

        返回：
            初始化完成的运行器实例

        异常：
            ValueError: 当运行器类型无效时抛出
        """
        runner_type = runner_type.lower()

        # 延迟导入以避免循环依赖
        if runner_type == "local":
            from aiotest.runners import LocalRunner
            runner = LocalRunner(user_types, load_shape, config)
        elif runner_type == "master":
            from aiotest.runners import MasterRunner
            runner = MasterRunner(user_types, load_shape, config, redis_client)
        elif runner_type == "worker":
            from aiotest.runners import WorkerRunner
            runner = WorkerRunner(user_types, load_shape, config, redis_client)
        else:
            raise ValueError(
                f"未知的运行器类型: {runner_type}。可用类型: local, master, worker")

        # 初始化运行器
        await runner.initialize()

        # 注册事件处理器
        registry = EventHandlerRegistry()
        await registry.register_handlers(runner)

        logger.info("已创建并初始化 %s 运行器", runner_type)
        return runner


class BaseRunner:
    """基础运行器，提供组合式架构的核心组件"""

    def __init__(self, user_types: List[Type['User']],
                 load_shape: Any, config: Dict[str, Any]):
        """
        初始化基础运行器

        参数：
            user_types: 用户类列表
            load_shape: 负载形状控制类
            config: 配置选项
        """
        # 基础属性
        self.user_types = user_types  # 保存用户类型列表
        self.load_shape = load_shape
        self.config = config
        self.node = None  # 节点类型，由子类设置
        self.cpu_usage = 0
        self.machine_id = None  # 机器标识符，用于区分不同机器上的Worker

        # 延迟初始化组件（避免循环导入）
        self._user_manager = None
        self._state_manager = None
        self._task_manager = None
        self._load_shape_manager = None

    @property
    def user_manager(self):
        if self._user_manager is None:
            self._user_manager = UserManager(self.user_types, self.config)
        return self._user_manager

    @property
    def state_manager(self):
        if self._state_manager is None:
            self._state_manager = StateManager()
        return self._state_manager

    @property
    def task_manager(self):
        if self._task_manager is None:
            self._task_manager = TaskManager()
        return self._task_manager

    @property
    def load_shape_manager(self):
        """获取负载形状管理器"""
        if self._load_shape_manager is None and self.load_shape:
            self._load_shape_manager = LoadShapeManager(
                self.load_shape, self.apply_load)
        return self._load_shape_manager

    async def initialize(self):
        """初始化运行器（由子类重写）"""
        import socket

        # 获取机器标识符（使用主机名）
        self.machine_id = socket.gethostname()

    async def start(self) -> None:
        """启动测试：从load_shape获取用户数和速率并执行测试"""
        if not self.load_shape:
            raise ValueError(
                "负载形状是必需的但未初始化。请检查运行器创建。")

        # 使用负载形状管理器启动测试
        await self.load_shape_manager.start()

    async def run_until_complete(self) -> None:
        """运行测试直到完成"""
        if self.load_shape_manager.task:
            await self.load_shape_manager.task

    async def quit(self):
        """退出运行器"""
        if self.state_manager.is_in_quit_state():
            return

        # 停止负载形状管理器
        if self._load_shape_manager:
            await self._load_shape_manager.stop()

        self.state_manager.set_quit_state()
        await self.state_manager.transition_state(RunnerState.QUITTING)

        try:
            # 停止所有用户
            await self.user_manager.stop_all_users()

            # 取消所有后台任务
            await self.task_manager.cancel_all_tasks()

            logger.info("%s 已成功退出", self.__class__.__name__)
        except Exception as e:
            logger.error("退出过程中出错: %s", str(e))
            raise

    async def pause(self) -> None:
        """暂停测试"""
        if not self.state_manager.can_pause():
            current_state = self.state_manager.get_current_state()
            logger.warning("无法从 %s 状态暂停，忽略暂停请求", current_state)
            return

        await self.state_manager.transition_state(RunnerState.PAUSED)
        # 暂停负载形状的计时
        if self.load_shape and hasattr(self.load_shape, 'pause'):
            self.load_shape.pause()
        # 暂停用户活动
        await self.user_manager.pause_all_users()
        logger.info("测试已暂停")

    async def resume(self) -> None:
        """恢复测试"""
        if not self.state_manager.can_resume():
            current_state = self.state_manager.get_current_state()
            logger.warning("无法从 %s 状态恢复，忽略恢复请求", current_state)
            return

        await self.state_manager.transition_state(RunnerState.RUNNING)
        # 恢复负载形状的计时
        if self.load_shape and hasattr(self.load_shape, 'resume'):
            self.load_shape.resume()
        # 恢复用户活动
        await self.user_manager.resume_all_users()
        logger.info("测试已恢复")

    @validate_params
    async def apply_load(self, user_count: int, rate: float) -> None:
        """应用负载配置：直接管理用户并更新状态"""
        # 初始状态处理
        if self.state_manager.can_start():
            await test_start.fire(runner=self)
            await self.state_manager.transition_state(RunnerState.STARTING)

        # 直接管理用户
        current_count = self.active_user_count
        if user_count > current_count:
            await self.user_manager.manage_users(user_count - current_count, rate, "start")
        elif user_count < current_count:
            await self.user_manager.manage_users(current_count - user_count, rate, "stop")
            self.user_manager.cleanup_inactive_users()

        # 首次启动设置运行状态
        if self.state_manager.get_current_state() == RunnerState.STARTING:
            await self.state_manager.transition_state(RunnerState.RUNNING)
            # 只有非Worker节点才触发全局启动完成事件
            if self.node != NODE_TYPE_WORKER:
                await startup_completed.fire(runner=self, node_type=self.node)

    async def stop(self) -> None:
        """停止负载测试（由子类重写）"""
        raise NotImplementedError("Subclass must implement stop method")

    @property
    def active_user_count(self) -> int:
        """获取当前活跃用户数量"""
        return self.user_manager.active_user_count

    @property
    def state(self):
        """获取当前状态"""
        return self.state_manager.get_current_state()

    async def _collect_cpu_metrics(self):
        """
        收集CPU使用率的通用方法

        适用于LocalRunner和WorkerRunner节点
        定期采集CPU使用率并更新self.cpu_usage属性
        """
        while not self.state_manager.is_in_quit_state():
            try:
                self.cpu_usage = psutil.cpu_percent(interval=1)

                # 性能预警
                if self.cpu_usage > 90:
                    logger.warning(
                        "%s CPU 使用率超过 90%%！(当前: %.1f%%)",
                        self.__class__.__name__, self.cpu_usage)
            except Exception as e:
                logger.error(
                    "%s 收集 CPU 指标失败: %s",
                    self.__class__.__name__, str(e))
            finally:
                await asyncio.sleep(5)


# ============================================================================
# 事件处理器方法 - 集中管理所有运行器相关的事件处理逻辑
# ============================================================================

async def on_startup_completed(node_type: str, **kwargs: Any) -> None:
    """启动完成事件处理"""
    runner = kwargs.get("runner")
    # 根据节点类型处理启动完成事件
    if node_type == NODE_TYPE_MASTER:
        # Master在所有Worker完成后才触发此事件，转换状态并记录汇总信息
        if runner and hasattr(runner, 'state_manager'):
            await runner.state_manager.transition_state(RunnerState.RUNNING)

        # 从启动完成跟踪器获取总用户数
        if runner and hasattr(runner, '_startup_completion_tracker'):
            tracker = runner._startup_completion_tracker
            total_users = tracker.get("startup_data", {}).get("user_count", 0)
            completed_workers = len(tracker.get("completed_workers", set()))
            expected_workers = tracker.get("expected_workers", 0)
            logger.info(
                "Master 启动完成: %d 个用户，共 %d/%d 个 Worker",
                total_users, completed_workers, expected_workers)
    elif node_type == NODE_TYPE_LOCAL:
        # LocalRunner
        if runner and hasattr(runner, 'state_manager'):
            await runner.state_manager.transition_state(RunnerState.RUNNING)
        if runner and hasattr(runner, 'active_user_count'):
            logger.info(
                "启动完成事件: 已启动 %d 个用户",
                runner.active_user_count)
    # Worker不在此处理startup_completed事件，避免死锁


async def on_worker_request_metrics(node_type: str, **kwargs: Any) -> None:
    """处理接收到的工作节点请求指标数据事件"""
    # 只在Master节点处理Worker请求指标
    if node_type != NODE_TYPE_MASTER:
        return

    runner = kwargs.get("runner")
    data = kwargs.get("data")
    if runner is None or not data:
        return

    collector = getattr(runner, 'metrics_collector', None)

    if collector:
        # 从字典创建 RequestMetrics 对象
        metrics = RequestMetrics(
            request_id=data.get("request_id", ""),
            method=data.get("method", "unknown"),
            endpoint=data.get("endpoint", "unknown"),
            status_code=data.get("status_code", 0),
            duration=data.get("duration", 0),
            response_size=data.get("response_size", 0),
            error=data.get("error", None)
        )
        await collector.process_request_metrics(metrics=metrics)



# encoding: utf-8

import os
import tempfile

import allure
import pytest

from aiotest.logger import ExtraFieldFormatter, Logger, logger


@allure.feature("ExtraFieldFormatter")
class TestExtraFieldFormatter:
    """ExtraFieldFormatter 类的测试用例"""

    @allure.story("初始化")
    @allure.title("测试 ExtraFieldFormatter 初始化")
    @allure.severity(allure.severity_level.NORMAL)
    def test_initialization(self):
        """测试 ExtraFieldFormatter 初始化时正确设置属性"""
        default_fmt = "%(asctime)s | %(levelname)s | %(message)s"
        field_fmt_dict = {
            "request_start": "%(asctime)s | %(levelname)s | %(message)s | %(request_start)s"
        }
        formatter = ExtraFieldFormatter(default_fmt, field_fmt_dict)
        assert formatter.default_fmt == default_fmt
        assert formatter.field_fmt_dict == field_fmt_dict

    @allure.story("格式化")
    @allure.title("测试默认格式")
    @allure.severity(allure.severity_level.NORMAL)
    def test_format_default(self):
        """测试使用默认格式"""
        import logging
        default_fmt = "%(asctime)s | %(levelname)s | %(message)s"
        field_fmt_dict = {
            "request_start": "%(asctime)s | %(levelname)s | %(message)s | %(request_start)s"
        }
        formatter = ExtraFieldFormatter(default_fmt, field_fmt_dict)

        record = logging.makeLogRecord(
            {"levelname": "INFO", "msg": "Test message"})
        formatted = formatter.format(record)
        assert "Test message" in formatted

    @allure.story("格式化")
    @allure.title("测试字段特定格式")
    @allure.severity(allure.severity_level.NORMAL)
    def test_format_with_field(self):
        """测试使用字段特定格式"""
        import logging
        default_fmt = "%(asctime)s | %(levelname)s | %(message)s"
        field_fmt_dict = {
            "request_start": "%(asctime)s | %(levelname)s | %(message)s | %(request_start)s"
        }
        formatter = ExtraFieldFormatter(default_fmt, field_fmt_dict)

        record = logging.makeLogRecord(
            {"levelname": "INFO", "msg": "Test message", "request_start": "2024-01-01 00:00:00"})
        formatted = formatter.format(record)
        assert "Test message" in formatted
        assert "2024-01-01 00:00:00" in formatted


@allure.feature("Logger")
class TestLogger:
    """Logger 类的测试用例"""

    @allure.story("初始化")
    @allure.title("测试 Logger 初始化")
    @allure.severity(allure.severity_level.NORMAL)
    def test_initialization(self):
        """测试 Logger 初始化时正确设置属性"""
        import logging
        with tempfile.TemporaryDirectory() as temp_dir:
            logger_instance = Logger(
                name="test_log",
                log_dir=temp_dir,
                log_level="DEBUG")
            assert logger_instance.name == "test_log"
            assert logger_instance.log_dir == temp_dir
            assert logger_instance.log_level == "DEBUG"
            assert logger_instance.logger is not None

            # 关闭日志处理器，释放文件锁
            for handler in logger_instance.logger.handlers:
                handler.close()
                logger_instance.logger.removeHandler(handler)

            # 清理日志记录器缓存
            if "test_log" in logging.Logger.manager.loggerDict:
                del logging.Logger.manager.loggerDict["test_log"]

    @allure.story("初始化")
    @allure.title("测试 Logger 重复初始化")
    @allure.severity(allure.severity_level.NORMAL)
    def test_initialization_repeated(self):
        """测试 Logger 重复初始化"""
        import logging
        with tempfile.TemporaryDirectory() as temp_dir:
            logger_instance = Logger(name="test_log", log_dir=temp_dir)
            # 再次初始化，应该不会重复添加处理器
            logger_instance._setup_logging()
            assert len(logger_instance.logger.handlers) == 2  # 文件处理器和标准输出处理器

            # 关闭日志处理器，释放文件锁
            for handler in logger_instance.logger.handlers:
                handler.close()
                logger_instance.logger.removeHandler(handler)

            # 清理日志记录器缓存
            if "test_log" in logging.Logger.manager.loggerDict:
                del logging.Logger.manager.loggerDict["test_log"]

    @allure.story("日志级别")
    @allure.title("测试设置日志级别")
    @allure.severity(allure.severity_level.NORMAL)
    def test_set_level(self):
        """测试设置日志级别"""
        import logging
        with tempfile.TemporaryDirectory() as temp_dir:
            logger_instance = Logger(
                name="test_log",
                log_dir=temp_dir,
                log_level="INFO")
            assert logger_instance.logger.level == 20  # INFO 级别

            logger_instance.setLevel("DEBUG")
            assert logger_instance.logger.level == 10  # DEBUG 级别

            # 关闭日志处理器，释放文件锁
            for handler in logger_instance.logger.handlers:
                handler.close()
                logger_instance.logger.removeHandler(handler)

            # 清理日志记录器缓存
            if "test_log" in logging.Logger.manager.loggerDict:
                del logging.Logger.manager.loggerDict["test_log"]

    @allure.story("日志级别")
    @allure.title("测试设置日志级别别名")
    @allure.severity(allure.severity_level.NORMAL)
    def test_set_level_alias(self):
        """测试设置日志级别（使用别名方法）"""
        import logging
        with tempfile.TemporaryDirectory() as temp_dir:
            logger_instance = Logger(
                name="test_log",
                log_dir=temp_dir,
                log_level="INFO")
            assert logger_instance.logger.level == 20  # INFO 级别

            logger_instance.set_level("DEBUG")
            assert logger_instance.logger.level == 10  # DEBUG 级别

            # 关闭日志处理器，释放文件锁
            for handler in logger_instance.logger.handlers:
                handler.close()
                logger_instance.logger.removeHandler(handler)

            # 清理日志记录器缓存
            if "test_log" in logging.Logger.manager.loggerDict:
                del logging.Logger.manager.loggerDict["test_log"]

    @allure.story("日志记录")
    @allure.title("测试记录 DEBUG 级别日志")
    @allure.severity(allure.severity_level.NORMAL)
    def test_debug(self):
        """测试记录 DEBUG 级别日志"""
        import logging
        with tempfile.TemporaryDirectory() as temp_dir:
            logger_instance = Logger(
                name="test_log",
                log_dir=temp_dir,
                log_level="DEBUG")
            logger_instance.debug("Debug message")
            # 这里我们只测试方法调用，不验证日志内容

            # 关闭日志处理器，释放文件锁
            for handler in logger_instance.logger.handlers:
                handler.close()
                logger_instance.logger.removeHandler(handler)

            # 清理日志记录器缓存
            if "test_log" in logging.Logger.manager.loggerDict:
                del logging.Logger.manager.loggerDict["test_log"]

    @allure.story("日志记录")
    @allure.title("测试记录 INFO 级别日志")
    @allure.severity(allure.severity_level.NORMAL)
    def test_info(self):
        """测试记录 INFO 级别日志"""
        import logging
        with tempfile.TemporaryDirectory() as temp_dir:
            logger_instance = Logger(name="test_log", log_dir=temp_dir)
            logger_instance.info("Info message")
            # 这里我们只测试方法调用，不验证日志内容

            # 关闭日志处理器，释放文件锁
            for handler in logger_instance.logger.handlers:
                handler.close()
                logger_instance.logger.removeHandler(handler)

            # 清理日志记录器缓存
            if "test_log" in logging.Logger.manager.loggerDict:
                del logging.Logger.manager.loggerDict["test_log"]

    @allure.story("日志记录")
    @allure.title("测试记录 WARNING 级别日志")
    @allure.severity(allure.severity_level.NORMAL)
    def test_warning(self):
        """测试记录 WARNING 级别日志"""
        import logging
        with tempfile.TemporaryDirectory() as temp_dir:
            logger_instance = Logger(name="test_log", log_dir=temp_dir)
            logger_instance.warning("Warning message")
            # 这里我们只测试方法调用，不验证日志内容

            # 关闭日志处理器，释放文件锁
            for handler in logger_instance.logger.handlers:
                handler.close()
                logger_instance.logger.removeHandler(handler)

            # 清理日志记录器缓存
            if "test_log" in logging.Logger.manager.loggerDict:
                del logging.Logger.manager.loggerDict["test_log"]

    @allure.story("日志记录")
    @allure.title("测试记录 ERROR 级别日志")
    @allure.severity(allure.severity_level.NORMAL)
    def test_error(self):
        """测试记录 ERROR 级别日志"""
        import logging
        with tempfile.TemporaryDirectory() as temp_dir:
            logger_instance = Logger(name="test_log", log_dir=temp_dir)
            logger_instance.error("Error message")
            # 这里我们只测试方法调用，不验证日志内容

            # 关闭日志处理器，释放文件锁
            for handler in logger_instance.logger.handlers:
                handler.close()
                logger_instance.logger.removeHandler(handler)

            # 清理日志记录器缓存
            if "test_log" in logging.Logger.manager.loggerDict:
                del logging.Logger.manager.loggerDict["test_log"]

    @allure.story("日志记录")
    @allure.title("测试记录 CRITICAL 级别日志")
    @allure.severity(allure.severity_level.NORMAL)
    def test_critical(self):
        """测试记录 CRITICAL 级别日志"""
        import logging
        with tempfile.TemporaryDirectory() as temp_dir:
            logger_instance = Logger(name="test_log", log_dir=temp_dir)
            logger_instance.critical("Critical message")
            # 这里我们只测试方法调用，不验证日志内容

            # 关闭日志处理器，释放文件锁
            for handler in logger_instance.logger.handlers:
                handler.close()
                logger_instance.logger.removeHandler(handler)

            # 清理日志记录器缓存
            if "test_log" in logging.Logger.manager.loggerDict:
                del logging.Logger.manager.loggerDict["test_log"]

    @allure.story("日志记录")
    @allure.title("测试获取日志记录器")
    @allure.severity(allure.severity_level.NORMAL)
    def test_get_logger(self):
        """测试获取日志记录器"""
        import logging
        with tempfile.TemporaryDirectory() as temp_dir:
            logger_instance = Logger(name="test_log", log_dir=temp_dir)
            log = logger_instance.get_logger()
            assert log is not None

            # 关闭日志处理器，释放文件锁
            for handler in logger_instance.logger.handlers:
                handler.close()
                logger_instance.logger.removeHandler(handler)

            # 清理日志记录器缓存
            if "test_log" in logging.Logger.manager.loggerDict:
                del logging.Logger.manager.loggerDict["test_log"]


@allure.feature("Global Logger")
class TestGlobalLogger:
    """全局日志记录器的测试用例"""

    @allure.story("全局日志")
    @allure.title("测试全局日志记录器")
    @allure.severity(allure.severity_level.NORMAL)
    def test_global_logger(self):
        """测试全局日志记录器"""
        assert logger is not None

    @allure.story("全局日志")
    @allure.title("测试全局日志记录")
    @allure.severity(allure.severity_level.NORMAL)
    def test_global_logger_logging(self):
        """测试使用全局日志记录器记录日志"""
        logger.info("Test global logger")
        # 这里我们只测试方法调用，不验证日志内容


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



# encoding: utf-8

import allure
import pytest

from aiotest import HTTPClient, configure_connector
from aiotest.clients import CONNECTOR_SETTINGS, ResponseContextManager
from aiotest.metrics import RequestMetrics


@allure.feature("HTTPClient")
class TestHTTPClient:
    """HTTPClient 类的测试用例"""

    @allure.story("初始化")
    @allure.title("测试 HTTPClient 初始化")
    @allure.severity(allure.severity_level.CRITICAL)
    def test_initialization(self):
        client = HTTPClient(
            base_url="http://api.example.com",
            default_headers={"X-Custom": "value"},
            timeout=60,
            max_retries=5,
            verify_ssl=True
        )
        assert client.base_url == "http://api.example.com"
        assert client.default_headers.get("X-Custom") == "value"
        assert client.default_headers.get("Accept") == "application/json"

    @allure.story("配置管理")
    @allure.title("测试配置连接池")
    @allure.severity(allure.severity_level.NORMAL)
    def test_configure_connector(self):
        original_limit = CONNECTOR_SETTINGS['limit']

        configure_connector(limit=200, limit_per_host=50)
        assert CONNECTOR_SETTINGS['limit'] == 200
        assert CONNECTOR_SETTINGS['limit_per_host'] == 50

        configure_connector(limit=original_limit)

    @allure.story("端点处理")
    @allure.title("测试标准化端点")
    @allure.severity(allure.severity_level.NORMAL)
    @pytest.mark.parametrize("endpoint, expected", [
        ("users/123", "users/{id}"),
        ("users/123/profile", "users/{id}/profile"),
        ("api/v1/users/456", "api/v1/users/{id}"),
        ("simple", "simple"),
        ("nested/path/test/789", "nested/path/test/{id}"),
    ])
    def test_normalize_endpoint(self, endpoint, expected):
        """测试标准化端点，包括正常路径和边界情况"""
        client = HTTPClient()
        assert client._normalize_endpoint(endpoint) == expected

    @allure.story("安全处理")
    @allure.title("测试清理敏感信息的请求头")
    @allure.severity(allure.severity_level.CRITICAL)
    def test_sanitize_headers(self):
        client = HTTPClient()
        headers = {
            "Authorization": "Bearer token123",
            "Content-Type": "application/json",
            "API-Key": "secret456"
        }
        sanitized = client._sanitize_headers(headers)
        assert sanitized["Authorization"] == "*****"
        assert sanitized["Content-Type"] == "application/json"
        assert sanitized["API-Key"] == "*****"

    @allure.story("安全处理")
    @allure.title("测试清理敏感信息的请求体")
    @allure.severity(allure.severity_level.CRITICAL)
    def test_sanitize_body(self):
        client = HTTPClient()
        body_dict = {
            "username": "test",
            "password": "secret123",
            "token": "abc123"
        }
        sanitized_dict = client._sanitize_body(body_dict)
        assert sanitized_dict["username"] == "test"
        assert sanitized_dict["password"] == "*****"
        assert sanitized_dict["token"] == "*****"

        body_str = "username=test&password=secret123&token=abc123"
        sanitized_str = client._sanitize_body(body_str)
        assert "password=*****" in sanitized_str
        assert "token=*****" in sanitized_str

    @allure.story("初始化")
    @allure.title("测试初始化参数验证")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_initialization_validation(self):
        async with HTTPClient(base_url="http://localhost:8080") as client:
            assert client.base_url == "http://localhost:8080"
            assert client.max_retries == 3
            assert client.verify_ssl is True

    @allure.story("生命周期管理")
    @allure.title("测试 close 方法")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_close_method(self):
        client = HTTPClient(base_url="http://localhost:8080")
        await client.__aenter__()
        assert client._session is not None
        await client.close()

    @allure.story("HTTP 请求")
    @allure.title("测试 HTTP 方法")
    @allure.severity(allure.severity_level.CRITICAL)
    async def test_http_methods(self):
        async with HTTPClient(base_url="http://localhost:8080") as http_client:
            async with http_client.get("/") as response:
                assert response.status == 200
                json_data = await response.json()
                assert json_data == {"message": "Hello, HTTP!"}

            async with http_client.post("/", data={"key": "value"}) as response:
                assert response.status == 200
                json_data = await response.json()
                assert json_data == {"message": "Hello, HTTP!"}

            async with http_client.put("/", data={"key": "value"}) as response:
                assert response.status == 200
                json_data = await response.json()
                assert json_data == {"message": "Hello, HTTP!"}

            async with http_client.delete("/") as response:
                assert response.status == 200
                json_data = await response.json()
                assert json_data == {"message": "Hello, HTTP!"}

    @allure.story("错误处理")
    @allure.title("测试错误处理")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_error_handling(self):
        async with HTTPClient(base_url="http://localhost:8080") as http_client:
            async with http_client.get("/error") as response:
                assert response.status == 500

    @allure.story("重试机制")
    @allure.title("测试重试逻辑")
    @allure.severity(allure.severity_level.CRITICAL)
    async def test_retry_logic(self):
        async with HTTPClient(base_url="http://localhost:8080", max_retries=3) as client:
            response = await client._request_with_retry('GET', 'retry')
            assert response.status == 200
            response.close()

    @allure.story("重试机制")
    @allure.title("测试 max_retries=0 时跳过重试逻辑")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_no_retry_when_max_retries_zero(self):
        """测试 max_retries=0 时跳过重试逻辑"""
        async with HTTPClient(base_url="http://localhost:8080", max_retries=0) as client:
            response = await client._request_with_retry('GET', '/')
            assert response.status == 200
            response.close()

    @allure.story("网络请求")
    @allure.title("测试 HTTPS 请求")
    @allure.severity(allure.severity_level.CRITICAL)
    async def test_https_request(self):
        async with HTTPClient(base_url="https://localhost:8443", verify_ssl=False) as https_client:
            async with https_client.get("/") as response:
                assert response.status == 200
                json_data = await response.json()
                assert json_data == {"message": "Hello, HTTPS!"}

    @allure.story("网络请求")
    @allure.title("测试真实 HTTP 请求")
    @allure.severity(allure.severity_level.CRITICAL)
    async def test_real_http_request(self):
        async with HTTPClient(base_url="http://localhost:8080") as client:
            async with client.get("/") as response:
                assert response.status == 200
                json_data = await response.json()
                assert json_data == {"message": "Hello, HTTP!"}

    @allure.story("网络请求")
    @allure.title("测试真实 HTTPS 请求")
    @allure.severity(allure.severity_level.CRITICAL)
    async def test_real_https_request(self):
        async with HTTPClient(base_url="https://localhost:8443", verify_ssl=False) as client:
            async with client.get("/") as response:
                assert response.status == 200
                json_data = await response.json()
                assert json_data == {"message": "Hello, HTTPS!"}

    @allure.story("功能特性")
    @allure.title("测试 name 参数功能")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_name_parameter(self):
        async with HTTPClient(base_url="http://localhost:8080") as client:
            async with client.get("/", name="get_root") as response:
                assert response.status == 200
                json_data = await response.json()
                assert json_data == {"message": "Hello, HTTP!"}

            async with client.get("/") as response:
                assert response.status == 200
                json_data = await response.json()
                assert json_data == {"message": "Hello, HTTP!"}

    @allure.story("性能数据")
    @allure.title("测试性能数据记录功能")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_record_request_metrics(self, http_client):
        """测试性能数据记录功能"""
        # 直接调用 _record_request_metrics 方法测试
        await http_client._record_request_metrics(
            request_id="test-req-1",
            method="GET",
            endpoint="/test",
            status_code=200,
            duration=0.1,
            response_size=100
        )
        # 这里主要测试方法是否能正常执行，不抛出异常
        assert True

    @allure.story("日志记录")
    @allure.title("测试日志记录功能")
    @allure.severity(allure.severity_level.NORMAL)
    def test_log_request(self):
        """测试日志记录功能"""
        client = HTTPClient()
        log = RequestMetrics(
            request_id="test-req-1",
            method="GET",
            endpoint="/test"
        )
        # 测试不同状态的日志记录
        client._log_request(log, "start")
        client._log_request(log, "complete")
        # 测试失败状态的日志记录
        log.error = {"message": "Test error"}
        client._log_request(log, "failed")
        # 这里主要测试方法是否能正常执行，不抛出异常
        assert True

    @allure.story("HTTP 请求")
    @allure.title("测试 request 方法的完整逻辑")
    @allure.severity(allure.severity_level.CRITICAL)
    async def test_request_method(self, http_client):
        """测试 request 方法的完整逻辑"""
        # 测试 request 方法
        async with http_client.request("GET", "/") as response:
            assert response.status == 200
            json_data = await response.json()
            assert json_data == {"message": "Hello, HTTP!"}

    @allure.story("重试机制")
    @allure.title("测试重试机制中 max_retries=0 的情况")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_request_with_retry_edge_cases(self):
        """测试重试机制中 max_retries=0 的情况"""
        # 测试 max_retries=0 的情况
        async with HTTPClient(base_url="http://localhost:8080", max_retries=0) as client:
            response = await client._request_with_retry('GET', '/')
            assert response.status == 200
            response.close()

    @allure.story("重试机制")
    @allure.title("测试当遇到特定状态码时放弃重试的逻辑")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_request_with_retry_giveup(self):
        """测试当遇到特定状态码时放弃重试的逻辑"""
        # 这里需要一个返回 404 状态码的端点
        async with HTTPClient(base_url="http://localhost:8080", max_retries=3) as client:
            # 尝试访问一个不存在的端点，应该返回 404 并放弃重试
            response = await client._request_with_retry('GET', '/nonexistent')
            assert response.status == 404
            response.close()


@allure.feature("ResponseContextManager")
class TestResponseContextManager:
    """ResponseContextManager 类的测试用例"""

    @allure.story("响应处理")
    @allure.title("测试响应上下文管理器")
    @allure.severity(allure.severity_level.CRITICAL)
    async def test_response_context_manager(self):
        async with HTTPClient(base_url="http://localhost:8080") as client:
            async with client.get("/") as response:
                assert response.status == 200
                json_data = await response.json()
                assert json_data == {"message": "Hello, HTTP!"}

    @allure.story("响应处理")
    @allure.title("测试 ResponseContextManager 的属性和方法")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_properties_and_methods(self, http_client):
        """测试 ResponseContextManager 的 status 属性和 json() 方法"""
        async with http_client.get("/") as response:
            assert response.status == 200

            # 测试 status 属性
            assert isinstance(response.status, int)

            # 测试 json() 方法
            json_data = await response.json()
            assert json_data == {"message": "Hello, HTTP!"}

    @allure.story("数据处理")
    @allure.title("测试 _estimate_size 方法")
    @allure.severity(allure.severity_level.NORMAL)
    def test_estimate_size(self):
        assert ResponseContextManager._estimate_size(None) == 0
        assert ResponseContextManager._estimate_size("test") == 4
        assert ResponseContextManager._estimate_size({"key": "value"}) == 16
        assert ResponseContextManager._estimate_size([1, 2, 3]) == 9
        assert ResponseContextManager._estimate_size(b"bytes") == 8

    @allure.story("数据处理")
    @allure.title("测试 _safe_get_headers 方法")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_safe_get_headers(self):
        async with HTTPClient(base_url="http://localhost:8080") as client:
            async with client.get("/custom-headers") as response:
                assert response.status == 200
                json_data = await response.json()
                assert json_data == {"message": "Hello with custom headers!"}
                headers = response._processed_data['headers']
                assert 'X-Custom-Header' in headers
                assert headers['X-Custom-Header'] == 'custom-value'

    @allure.story("数据处理")
    @allure.title("测试 _safe_get_cookies 方法")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_safe_get_cookies(self):
        async with HTTPClient(base_url="http://localhost:8080") as client:
            async with client.get("/cookies") as response:
                assert response.status == 200
                json_data = await response.json()
                assert json_data == {"message": "Hello with cookies!"}
                cookies = response._processed_data['cookies']
                assert 'session_id' in cookies or 'user_id' in cookies

    @allure.story("数据处理")
    @allure.title("测试 _safe_get_cookies 方法处理空 cookies")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_safe_get_cookies_empty(self):
        async with HTTPClient(base_url="http://localhost:8080") as client:
            async with client.get("/") as response:
                assert response.status == 200
                cookies = response._processed_data['cookies']
                assert cookies == {}

    @allure.story("响应处理")
    @allure.title("测试 _process_response 方法处理 JSON 解析失败的情况")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_process_response_json_error(self):
        async with HTTPClient(base_url="http://localhost:8080") as client:
            async with client.get("/text") as response:
                result = await response._process_response(response._response)
                assert result['status'] == 200
                assert result['data'] == "This is plain text response"

    @allure.story("响应处理")
    @allure.title("测试 _process_response 方法处理异常情况")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_process_response_exception(self):
        """测试 _process_response 方法处理异常情况"""
        async with HTTPClient(base_url="http://localhost:8080") as client:
            async with client.get("/error") as response:
                assert response.status == 500
                text = await response.text()
                assert "Internal Server Error" in text

    @allure.story("响应处理")
    @allure.title("测试响应上下文管理器处理异常")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_context_manager_exception_handling(self):
        """测试响应上下文管理器在异常情况下的处理"""
        async with HTTPClient(base_url="http://localhost:8080") as client:
            try:
                async with client.get("/error") as response:
                    # 验证状态码为 500
                    assert response.status == 500
            except Exception:
                # 如果发生异常，确保客户端仍然可用
                pass

    @allure.story("数据处理")
    @allure.title("测试 _estimate_size 方法的边界情况")
    @allure.severity(allure.severity_level.NORMAL)
    def test_estimate_size_boundary(self):
        """测试 _estimate_size 方法的边界情况"""
        # 测试空值
        assert ResponseContextManager._estimate_size(None) == 0
        # 测试空字符串
        assert ResponseContextManager._estimate_size("") == 0
        # 测试空字典（转换为字符串后为"{}"，编码后长度为2）
        assert ResponseContextManager._estimate_size({}) == 2
        # 测试空列表（转换为字符串后为"[]"，编码后长度为2）
        assert ResponseContextManager._estimate_size([]) == 2
        # 测试字节串（str(b"test") = "b'test'"，编码后长度为7）
        assert ResponseContextManager._estimate_size(b"test") == 7
        # 测试嵌套字典
        nested_dict = {"key": {"nested": "value"}}
        assert ResponseContextManager._estimate_size(nested_dict) > 0

    @allure.story("错误处理")
    @allure.title("测试错误处理功能")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_handle_error(self):
        """测试错误处理功能"""
        # 创建一个简单的 ResponseContextManager 实例
        async def mock_request():
            raise Exception("Test error")

        log = RequestMetrics(
            request_id="test-req-1",
            method="GET",
            endpoint="/test"
        )

        async def mock_metrics_callback(*args, **kwargs):
            pass

        def mock_log_request(*args, **kwargs):
            pass

        rcm = ResponseContextManager(
            request_coroutine=mock_request(),
            log=log,
            start_time=0,
            metrics_callback=mock_metrics_callback,
            log_request=mock_log_request
        )

        # 测试错误处理
        with pytest.raises(Exception):
            async with rcm as response:
                pass

    @allure.story("上下文管理")
    @allure.title("测试上下文管理器在错误情况下的退出逻辑")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_context_manager_exit_with_error(self):
        """测试上下文管理器在错误情况下的退出逻辑"""
        async with HTTPClient(base_url="http://localhost:8080") as client:
            try:
                async with client.get("/error") as response:
                    # 验证状态码为 500
                    assert response.status == 500
            except Exception:
                # 如果发生异常，确保客户端仍然可用
                pass

    @allure.story("上下文管理")
    @allure.title("测试上下文管理器的进入逻辑")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_context_manager_enter(self, http_client):
        """测试上下文管理器的进入逻辑"""
        async with http_client.get("/") as response:
            # 验证响应对象被正确创建
            assert response is not None
            assert response.status == 200

    @allure.story("边界测试")
    @allure.title("测试空响应体的处理")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_empty_response(self):
        """测试空响应体的处理"""
        # 这里需要一个返回空响应的端点
        # 暂时使用现有的端点
        async with HTTPClient(base_url="http://localhost:8080") as client:
            async with client.get("/") as response:
                assert response.status == 200

    @allure.story("边界测试")
    @allure.title("测试超大响应体的处理")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_large_response(self):
        """测试超大响应体的处理"""
        # 这里需要一个返回大响应的端点
        # 暂时使用现有的端点
        async with HTTPClient(base_url="http://localhost:8080") as client:
            async with client.get("/") as response:
                assert response.status == 200

    @allure.story("边界测试")
    @allure.title("测试网络超时的处理")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_network_timeout(self):
        """测试网络超时的处理"""
        # 创建一个超时时间很短的客户端
        async with HTTPClient(base_url="http://localhost:8080", timeout=0.001) as client:
            with pytest.raises(Exception):
                async with client.get("/") as response:
                    pass

    @allure.story("边界测试")
    @allure.title("测试连接被重置的情况")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_connection_reset(self):
        """测试连接被重置的情况"""
        async with HTTPClient(base_url="http://localhost:8080") as client:
            # 尝试访问会导致连接重置的端点
            with pytest.raises(Exception):
                async with client.get("/reset-connection") as response:
                    pass

            # 验证客户端在连接被重置后仍然可用
            async with client.get("/") as response:
                assert response.status == 200

    @allure.story("响应处理")
    @allure.title("测试响应处理的完整逻辑")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_aenter_response_processing(self, http_client):
        """测试响应处理的完整逻辑"""
        async with http_client.get("/") as response:
            # 验证响应对象被正确创建
            assert response is not None
            assert response.status == 200
            # 验证处理后的数据
            assert hasattr(response, "_processed_data")
            assert "status" in response._processed_data
            assert "headers" in response._processed_data
            assert "data" in response._processed_data
            assert "cookies" in response._processed_data

    @allure.story("响应处理")
    @allure.title("测试 headers 属性")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_headers_property(self, http_client):
        """测试 headers 属性能正确返回响应头"""
        async with http_client.get("/") as response:
            assert response.status == 200
            # 测试 headers 属性
            headers = response.headers
            assert isinstance(headers, dict)
            assert "Content-Type" in headers

    @allure.story("错误处理")
    @allure.title("测试 _handle_error 方法的重复错误处理检查")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_handle_error_no_duplication(self):
        """测试 _handle_error 方法的重复错误处理检查"""
        # 创建一个简单的 ResponseContextManager 实例
        async def mock_request():
            raise Exception("Test error")

        log = RequestMetrics(
            request_id="test-req-1",
            method="GET",
            endpoint="/test"
        )

        async def mock_metrics_callback(*args, **kwargs):
            pass

        def mock_log_request(*args, **kwargs):
            pass

        rcm = ResponseContextManager(
            request_coroutine=mock_request(),
            log=log,
            start_time=0,
            metrics_callback=mock_metrics_callback,
            log_request=mock_log_request
        )

        # 第一次调用 _handle_error
        await rcm._handle_error(Exception("Test error"))

        # 验证错误已被标记为处理
        assert hasattr(rcm, "_error_handled")
        assert rcm._error_handled is True

        # 第二次调用 _handle_error，应该直接返回，不重复处理
        await rcm._handle_error(Exception("Test error"))

        # 验证错误仍然被标记为已处理
        assert rcm._error_handled is True

    @allure.story("错误处理")
    @allure.title("测试不同类型异常的处理")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_aexit_exception_handling(self):
        """测试不同类型异常的处理"""
        # 创建一个简单的 ResponseContextManager 实例
        async def mock_request():
            raise ValueError("Test value error")

        log = RequestMetrics(
            request_id="test-req-1",
            method="GET",
            endpoint="/test"
        )

        async def mock_metrics_callback(*args, **kwargs):
            pass

        def mock_log_request(*args, **kwargs):
            pass

        rcm = ResponseContextManager(
            request_coroutine=mock_request(),
            log=log,
            start_time=0,
            metrics_callback=mock_metrics_callback,
            log_request=mock_log_request
        )

        # 测试错误处理
        with pytest.raises(Exception):
            async with rcm as response:
                pass

    @allure.story("安全处理")
    @allure.title("测试清理请求头的边界情况")
    @allure.severity(allure.severity_level.NORMAL)
    def test_sanitize_headers_edge_cases(self):
        """测试清理请求头的边界情况"""
        client = HTTPClient()

        # 测试空字典
        assert client._sanitize_headers({}) == {}

        # 测试没有敏感信息的请求头
        headers = {"Content-Type": "application/json", "Accept": "*/*"}
        assert client._sanitize_headers(headers) == headers

        # 测试包含敏感信息的请求头
        sensitive_headers = {
            "Authorization": "Bearer token123",
            "Content-Type": "application/json",
            "API-Key": "secret456",
            "Cookie": "session_id=abc123"
        }
        sanitized = client._sanitize_headers(sensitive_headers)
        assert sanitized["Authorization"] == "*****"
        assert sanitized["Content-Type"] == "application/json"
        assert sanitized["API-Key"] == "*****"
        assert sanitized["Cookie"] == "*****"

    @allure.story("安全处理")
    @allure.title("测试清理请求体的边界情况")
    @allure.severity(allure.severity_level.NORMAL)
    def test_sanitize_body_edge_cases(self):
        """测试清理请求体的边界情况"""
        client = HTTPClient()

        # 测试 None
        assert client._sanitize_body(None) is None

        # 测试空字典
        assert client._sanitize_body({}) == {}

        # 测试空字符串
        assert client._sanitize_body("") == ""

        # 测试没有敏感信息的请求体
        body_dict = {"username": "test", "email": "test@example.com"}
        assert client._sanitize_body(body_dict) == body_dict

        # 测试包含敏感信息的请求体
        sensitive_body = {
            "username": "test",
            "password": "secret123",
            "token": "abc123"
        }
        sanitized = client._sanitize_body(sensitive_body)
        assert sanitized["username"] == "test"
        assert sanitized["password"] == "*****"
        assert sanitized["token"] == "*****"

    @allure.story("数据处理")
    @allure.title("测试处理文本类型的响应")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_process_response_edge_cases(self):
        """测试处理文本类型的响应"""
        # 测试文本类型的响应
        async with HTTPClient(base_url="http://localhost:8080") as client:
            async with client.get("/text") as response:
                assert response.status == 200
                text = await response.text()
                assert isinstance(text, str)

    @allure.story("错误处理")
    @allure.title("测试错误处理功能")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_handle_error_edge_cases(self):
        """测试错误处理功能"""
        # 创建一个简单的 ResponseContextManager 实例
        async def mock_request():
            raise Exception("Test error")

        log = RequestMetrics(
            request_id="test-req-1",
            method="GET",
            endpoint="/test"
        )

        async def mock_metrics_callback(*args, **kwargs):
            pass

        def mock_log_request(*args, **kwargs):
            pass

        rcm = ResponseContextManager(
            request_coroutine=mock_request(),
            log=log,
            start_time=0,
            metrics_callback=mock_metrics_callback,
            log_request=mock_log_request
        )

        # 测试错误处理
        with pytest.raises(Exception):
            async with rcm as response:
                pass

    @allure.story("数据处理")
    @allure.title("测试安全获取 headers 的边界情况")
    @allure.severity(allure.severity_level.NORMAL)
    def test_safe_get_headers_edge_cases(self):
        """测试安全获取 headers 的边界情况"""
        # 测试 None
        assert ResponseContextManager._safe_get_headers(
            None) == {"Content-Type": "application/json"}

        # 测试没有 headers 属性的对象
        class MockResponse:
            pass
        mock_response = MockResponse()
        assert ResponseContextManager._safe_get_headers(
            mock_response) == {"Content-Type": "application/json"}

    @allure.story("数据处理")
    @allure.title("测试安全获取 cookies 的边界情况")
    @allure.severity(allure.severity_level.NORMAL)
    def test_safe_get_cookies_edge_cases(self):
        """测试安全获取 cookies 的边界情况"""
        # 测试 None
        assert ResponseContextManager._safe_get_cookies(None) == {}

        # 测试没有 cookies 属性的对象
        class MockResponse:
            pass
        mock_response = MockResponse()
        assert ResponseContextManager._safe_get_cookies(mock_response) == {}

    @allure.story("配置管理")
    @allure.title("测试配置连接池的参数设置")
    @allure.severity(allure.severity_level.NORMAL)
    def test_configure_connector_edge_cases(self):
        """测试配置连接池的参数设置"""
        original_limit = CONNECTOR_SETTINGS['limit']
        original_limit_per_host = CONNECTOR_SETTINGS['limit_per_host']

        try:
            # 测试部分参数配置
            configure_connector(limit=150)
            assert CONNECTOR_SETTINGS['limit'] == 150
            assert CONNECTOR_SETTINGS['limit_per_host'] == original_limit_per_host

            # 测试所有参数配置
            configure_connector(
                limit=200,
                limit_per_host=50,
                force_close=True,
                enable_cleanup_closed=False)
            assert CONNECTOR_SETTINGS['limit'] == 200
            assert CONNECTOR_SETTINGS['limit_per_host'] == 50
            assert CONNECTOR_SETTINGS['force_close'] is True
            assert CONNECTOR_SETTINGS['enable_cleanup_closed'] is False
        finally:
            # 恢复原始配置
            configure_connector(
                limit=original_limit,
                limit_per_host=original_limit_per_host,
                force_close=False,
                enable_cleanup_closed=True)

    @allure.story("会话管理")
    @allure.title("测试会话关闭超时")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_session_close_timeout(self):
        """测试会话关闭超时的情况"""
        import asyncio

        class MockSession:
            async def close(self):
                await asyncio.sleep(10)  # 模拟关闭超时

        client = HTTPClient()
        client._session = MockSession()

        # 测试会话关闭超时
        await client.close()

    @allure.story("指标记录")
    @allure.title("测试指标记录失败")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_metrics_recording_failure(self):
        """测试记录请求指标失败的情况"""
        # 创建一个简单的 ResponseContextManager 实例
        async def mock_request():
            # 模拟一个响应
            class MockResponse:
                status = 200
                cookies = {}

                async def read(self):
                    return b"test"

                def close(self):
                    pass
            return MockResponse()

        log = RequestMetrics(
            request_id="test-req-1",
            method="GET",
            endpoint="/test"
        )

        # 模拟 metrics_callback 抛出异常
        async def failing_metrics_callback(**kwargs):
            raise Exception("Metrics recording failed")

        def mock_log_request(*args, **kwargs):
            pass

        rcm = ResponseContextManager(
            request_coroutine=mock_request(),
            log=log,
            start_time=0,
            metrics_callback=failing_metrics_callback,
            log_request=mock_log_request
        )

        # 测试进入上下文（会触发指标记录）
        try:
            async with rcm as response:
                pass
        except Exception:
            pass

    @allure.story("响应处理")
    @allure.title("测试响应处理失败")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_response_processing_failure(self):
        """测试处理响应数据失败的情况"""
        # 测试 _process_response 方法的异常处理
        # 注意：根据实际实现，_process_response 可能不会捕获所有异常
        # 我们测试的是该方法能够处理响应数据失败的情况
        class MockResponse:
            status = 200
            cookies = {}

            @property
            def headers(self):
                return {}

            async def read(self):
                return b"{invalid json}"  # 模拟JSON解析失败

            def close(self):
                pass

        # 创建一个简单的 ResponseContextManager 实例
        async def mock_request():
            return MockResponse()

        log = RequestMetrics(
            request_id="test-req-1",
            method="GET",
            endpoint="/test"
        )

        async def mock_metrics_callback(**kwargs):
            pass

        def mock_log_request(*args, **kwargs):
            pass

        rcm = ResponseContextManager(
            request_coroutine=mock_request(),
            log=log,
            start_time=0,
            metrics_callback=mock_metrics_callback,
            log_request=mock_log_request
        )

        # 测试处理响应（应该捕获JSON解析异常并返回默认值）
        result = await rcm._process_response(MockResponse())
        assert result is not None

    @allure.story("数据处理")
    @allure.title("测试安全获取Cookie异常")
    @allure.severity(allure.severity_level.NORMAL)
    def test_safe_get_cookies_exception(self):
        """测试安全获取Cookie时的异常处理"""
        # 测试会抛出异常的情况
        class MockResponse:
            @property
            def cookies(self):
                raise TypeError("Cookie processing failed")

        # 测试安全获取Cookie（应该捕获异常并返回空字典）
        result = ResponseContextManager._safe_get_cookies(MockResponse())
        assert result == {}

    @allure.story("日志处理")
    @allure.title("测试日志extra处理")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_log_extra_handling(self):
        """测试日志extra为None的处理"""
        # 模拟一个响应
        class MockResponse:
            status = 200
            cookies = {}

            async def read(self):
                return b"test"

            def close(self):
                pass

        # 创建一个log.extra为None的RequestMetrics
        log = RequestMetrics(
            request_id="test-req-1",
            method="GET",
            endpoint="/test"
        )
        log.extra = None

        # 创建一个简单的 ResponseContextManager 实例
        async def mock_request():
            return MockResponse()

        async def mock_metrics_callback(**kwargs):
            pass

        def mock_log_request(*args, **kwargs):
            pass

        rcm = ResponseContextManager(
            request_coroutine=mock_request(),
            log=log,
            start_time=0,
            metrics_callback=mock_metrics_callback,
            log_request=mock_log_request
        )

        # 测试进入上下文（应该将log.extra设置为空字典）
        async with rcm as response:
            pass

        # 验证 log.extra 不为 None
        assert rcm.log.extra is not None
        # 验证 log.extra 是一个字典
        assert isinstance(rcm.log.extra, dict)

    @allure.story("上下文管理")
    @allure.title("测试__aexit__异常处理")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_aexit_exception_handling(self):
        """测试__aexit__方法中的异常处理"""
        # 创建一个简单的 ResponseContextManager 实例
        async def mock_request():
            raise Exception("Test error")

        log = RequestMetrics(
            request_id="test-req-1",
            method="GET",
            endpoint="/test"
        )

        async def mock_metrics_callback(*args, **kwargs):
            pass

        def mock_log_request(*args, **kwargs):
            pass

        rcm = ResponseContextManager(
            request_coroutine=mock_request(),
            log=log,
            start_time=0,
            metrics_callback=mock_metrics_callback,
            log_request=mock_log_request
        )

        # 测试__aexit__方法处理异常的情况
        class TestException(Exception):
            pass

        # 调用__aexit__方法处理异常
        result = await rcm.__aexit__(TestException, TestException("Test error"), None)
        assert result is False

    @allure.story("日志记录")
    @allure.title("测试失败日志响应数据处理")
    @allure.severity(allure.severity_level.NORMAL)
    def test_failed_log_response_data_handling(self):
        """测试失败日志中的响应数据处理"""
        # 模拟一个包含错误信息的RequestMetrics
        log = RequestMetrics(
            request_id="test-req-1",
            method="GET",
            endpoint="/test"
        )
        log.error = {
            "response_data": "a" * 1000  # 测试长响应数据
        }

        # 直接测试 _log_request 方法的逻辑
        # 模拟日志数据构建
        log_data = {
            "method": log.method,
            "endpoint": log.endpoint,
            "status": "failed",
        }

        if hasattr(log, 'error'):
            log_data["error"] = log.error
            # 如果有响应数据,也记录到日志中
            if log.error.get("response_data") is not None:
                response_data = log.error["response_data"]
                # 限制响应数据长度
                if isinstance(response_data, (dict, str)):
                    import json
                    response_str = json.dumps(
                        response_data, ensure_ascii=False) if isinstance(
                        response_data, dict) else str(response_data)
                    if len(response_str) > 500:
                        response_str = response_str[:500] + "..."
                    log_data["response_data"] = response_str
                else:
                    log_data["response_data"] = str(response_data)[:500]

        # 验证响应数据被截断
        assert "response_data" in log_data
        assert len(log_data["response_data"]) <= 503  # 500 + "..."
        assert log_data["response_data"] == "a" * 500 + "..."

    @allure.story("断言处理")
    @allure.title("测试断言失败但状态码为200的情况")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_assertion_fail_with_200_status(self):
        """测试断言失败但HTTP状态码为200的情况"""
        # 模拟一个返回200状态码的响应
        async def mock_request():
            class MockResponse:
                status = 200
                cookies = {}

                async def read(self):
                    return b'{"message": "success"}'

                def close(self):
                    pass
            return MockResponse()

        log = RequestMetrics(
            request_id="test-req-1",
            method="GET",
            endpoint="/test"
        )

        # 用于验证断言结果的标志
        metrics_callback_called = False
        assertion_result_received = None

        async def mock_metrics_callback(**kwargs):
            nonlocal metrics_callback_called, assertion_result_received
            metrics_callback_called = True
            # 验证 assertion_result 被设置为 "fail"
            assertion_result_received = kwargs.get('assertion_result')

        def mock_log_request(*args, **kwargs):
            pass

        rcm = ResponseContextManager(
            request_coroutine=mock_request(),
            log=log,
            start_time=0,
            metrics_callback=mock_metrics_callback,
            log_request=mock_log_request
        )

        # 测试断言失败的情况
        try:
            async with rcm as response:
                # 断言失败，即使状态码是200
                assert False, "故意的断言失败"
        except AssertionError:
            # 预期的断言失败
            pass

        # 验证 assertion_result 被设置为 "fail"
        assert rcm.log.assertion_result == "fail"
        # 验证原始状态码被保留
        assert rcm.log.status_code == 200
        # 验证 metrics_callback 被调用
        assert metrics_callback_called
        # 验证 assertion_result 被正确传递
        assert assertion_result_received == "fail"

    @allure.story("断言处理")
    @allure.title("测试断言成功但状态码为500的情况")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_assertion_pass_with_500_status(self):
        """测试断言成功但HTTP状态码为500的情况"""
        # 模拟一个返回500状态码的响应
        async def mock_request():
            class MockResponse:
                status = 500
                cookies = {}

                async def read(self):
                    return b'{"message": "error"}'

                def close(self):
                    pass
            return MockResponse()

        log = RequestMetrics(
            request_id="test-req-1",
            method="GET",
            endpoint="/test"
        )

        # 用于验证断言结果的标志
        metrics_callback_called = False
        assertion_result_received = None

        async def mock_metrics_callback(**kwargs):
            nonlocal metrics_callback_called, assertion_result_received
            metrics_callback_called = True
            # 验证 assertion_result 被设置为 "pass"
            assertion_result_received = kwargs.get('assertion_result')

        def mock_log_request(*args, **kwargs):
            pass

        rcm = ResponseContextManager(
            request_coroutine=mock_request(),
            log=log,
            start_time=0,
            metrics_callback=mock_metrics_callback,
            log_request=mock_log_request
        )

        # 测试断言成功的情况
        async with rcm as response:
            # 断言成功，即使状态码是500
            assert True

        # 验证 assertion_result 被设置为 "pass"
        assert rcm.log.assertion_result == "pass"
        # 验证原始状态码被保留
        assert rcm.log.status_code == 500
        # 验证 metrics_callback 被调用
        assert metrics_callback_called
        # 验证 assertion_result 被正确传递
        assert assertion_result_received == "pass"

    @allure.story("错误处理")
    @allure.title("测试超时错误时的指标记录")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_timeout_error_metrics(self):
        """测试超时错误时的指标记录"""
        import asyncio

        # 模拟一个超时的请求
        async def mock_request():
            await asyncio.sleep(10)
            raise asyncio.TimeoutError("Request timeout")

        log = RequestMetrics(
            request_id="test-req-1",
            method="GET",
            endpoint="/test"
        )

        # 用于验证指标回调的参数
        metrics_callback_called = False
        assertion_result_received = None
        error_received = None

        async def mock_metrics_callback(**kwargs):
            nonlocal metrics_callback_called, assertion_result_received, error_received
            metrics_callback_called = True
            assertion_result_received = kwargs.get('assertion_result')
            error_received = kwargs.get('error')

        def mock_log_request(*args, **kwargs):
            pass

        rcm = ResponseContextManager(
            request_coroutine=mock_request(),
            log=log,
            start_time=0,
            metrics_callback=mock_metrics_callback,
            log_request=mock_log_request
        )

        # 测试超时错误
        with pytest.raises((asyncio.TimeoutError, Exception)):
            async with rcm as response:
                pass

        # 验证 assertion_result 被设置为 "fail"
        assert rcm.log.assertion_result == "fail"
        # 验证 metrics_callback 被调用
        assert metrics_callback_called
        # 验证 assertion_result 被正确传递
        assert assertion_result_received == "fail"
        # 验证错误信息被正确记录
        assert error_received is not None
        assert error_received.get('exc_type') in ['TimeoutError', 'Exception']

    @allure.story("错误处理")
    @allure.title("测试连接错误时的指标记录")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_connection_error_metrics(self):
        """测试连接错误时的指标记录"""
        import aiohttp

        # 模拟一个连接错误的请求
        async def mock_request():
            raise aiohttp.ClientConnectionError("Connection refused")

        log = RequestMetrics(
            request_id="test-req-1",
            method="GET",
            endpoint="/test"
        )

        # 用于验证指标回调的参数
        metrics_callback_called = False
        assertion_result_received = None
        error_received = None

        async def mock_metrics_callback(**kwargs):
            nonlocal metrics_callback_called, assertion_result_received, error_received
            metrics_callback_called = True
            assertion_result_received = kwargs.get('assertion_result')
            error_received = kwargs.get('error')

        def mock_log_request(*args, **kwargs):
            pass

        rcm = ResponseContextManager(
            request_coroutine=mock_request(),
            log=log,
            start_time=0,
            metrics_callback=mock_metrics_callback,
            log_request=mock_log_request
        )

        # 测试连接错误
        with pytest.raises((aiohttp.ClientConnectionError, Exception)):
            async with rcm as response:
                pass

        # 验证 assertion_result 被设置为 "fail"
        assert rcm.log.assertion_result == "fail"
        # 验证 metrics_callback 被调用
        assert metrics_callback_called
        # 验证 assertion_result 被正确传递
        assert assertion_result_received == "fail"
        # 验证错误信息被正确记录
        assert error_received is not None
        assert 'ConnectionError' in error_received.get('exc_type', '')

    @allure.story("错误处理")
    @allure.title("测试HTTP 4xx错误时的指标记录")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_http_4xx_error_metrics(self):
        """测试HTTP 4xx错误时的指标记录"""
        # 模拟一个返回404状态码的响应
        async def mock_request():
            class MockResponse:
                status = 404
                cookies = {}

                async def read(self):
                    return b'{"error": "Not Found"}'

                def close(self):
                    pass
            return MockResponse()

        log = RequestMetrics(
            request_id="test-req-1",
            method="GET",
            endpoint="/test"
        )

        # 用于验证指标回调的参数
        metrics_callback_called = False
        assertion_result_received = None
        status_code_received = None

        async def mock_metrics_callback(**kwargs):
            nonlocal metrics_callback_called, assertion_result_received, status_code_received
            metrics_callback_called = True
            assertion_result_received = kwargs.get('assertion_result')
            status_code_received = kwargs.get('status_code')

        def mock_log_request(*args, **kwargs):
            pass

        rcm = ResponseContextManager(
            request_coroutine=mock_request(),
            log=log,
            start_time=0,
            metrics_callback=mock_metrics_callback,
            log_request=mock_log_request
        )

        # 测试HTTP 4xx错误（断言失败）
        try:
            async with rcm as response:
                # 断言失败，即使状态码是404
                assert False, "HTTP 4xx error"
        except AssertionError:
            pass

        # 验证 assertion_result 被设置为 "fail"
        assert rcm.log.assertion_result == "fail"
        # 验证原始状态码被保留
        assert rcm.log.status_code == 404
        # 验证 metrics_callback 被调用
        assert metrics_callback_called
        # 验证 assertion_result 被正确传递
        assert assertion_result_received == "fail"
        # 验证状态码被正确传递
        assert status_code_received == 404

    @allure.story("错误处理")
    @allure.title("测试HTTP 5xx错误时的指标记录")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_http_5xx_error_metrics(self):
        """测试HTTP 5xx错误时的指标记录"""
        # 模拟一个返回500状态码的响应
        async def mock_request():
            class MockResponse:
                status = 500
                cookies = {}

                async def read(self):
                    return b'{"error": "Internal Server Error"}'

                def close(self):
                    pass
            return MockResponse()

        log = RequestMetrics(
            request_id="test-req-1",
            method="GET",
            endpoint="/test"
        )

        # 用于验证指标回调的参数
        metrics_callback_called = False
        assertion_result_received = None
        status_code_received = None
        error_received = None

        async def mock_metrics_callback(**kwargs):
            nonlocal metrics_callback_called, assertion_result_received, status_code_received, error_received
            metrics_callback_called = True
            assertion_result_received = kwargs.get('assertion_result')
            status_code_received = kwargs.get('status_code')
            error_received = kwargs.get('error')

        def mock_log_request(*args, **kwargs):
            pass

        rcm = ResponseContextManager(
            request_coroutine=mock_request(),
            log=log,
            start_time=0,
            metrics_callback=mock_metrics_callback,
            log_request=mock_log_request
        )

        # 测试HTTP 5xx错误（断言失败）
        try:
            async with rcm as response:
                # 断言失败，即使状态码是500
                assert False, "HTTP 5xx error"
        except AssertionError:
            pass

        # 验证 assertion_result 被设置为 "fail"
        assert rcm.log.assertion_result == "fail"
        # 验证原始状态码被保留
        assert rcm.log.status_code == 500
        # 验证 metrics_callback 被调用
        assert metrics_callback_called
        # 验证 assertion_result 被正确传递
        assert assertion_result_received == "fail"
        # 验证状态码被正确传递
        assert status_code_received == 500
        # 验证错误信息被正确记录
        assert error_received is not None
        assert 'AssertionError' in error_received.get('exc_type', '')

    @allure.story("错误处理")
    @allure.title("测试错误消息包含响应数据")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_error_message_includes_response_data(self):
        """测试错误消息包含响应数据"""
        # 模拟一个返回错误响应的请求
        async def mock_request():
            class MockResponse:
                status = 400
                cookies = {}

                async def read(self):
                    return b'{"error": "Bad Request", "details": "Invalid parameter"}'

                def close(self):
                    pass
            return MockResponse()

        log = RequestMetrics(
            request_id="test-req-1",
            method="POST",
            endpoint="/test"
        )

        # 用于验证错误消息
        error_message_received = None

        async def mock_metrics_callback(**kwargs):
            nonlocal error_message_received
            error = kwargs.get('error')
            if error:
                error_message_received = error.get('message', '')

        def mock_log_request(*args, **kwargs):
            pass

        rcm = ResponseContextManager(
            request_coroutine=mock_request(),
            log=log,
            start_time=0,
            metrics_callback=mock_metrics_callback,
            log_request=mock_log_request
        )

        # 测试错误消息包含响应数据
        try:
            async with rcm as response:
                # 断言失败
                assert False, "Bad Request"
        except AssertionError:
            pass

        # 验证错误消息包含响应数据
        assert error_message_received is not None
        assert "Response:" in error_message_received or "error" in error_message_received

    @allure.story("错误处理")
    @allure.title("测试断言成功但HTTP错误时的指标记录")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_assertion_pass_with_http_error(self):
        """测试断言成功但HTTP错误时的指标记录"""
        # 模拟一个返回500状态码的响应
        async def mock_request():
            class MockResponse:
                status = 500
                cookies = {}

                async def read(self):
                    return b'{"error": "Internal Server Error"}'

                def close(self):
                    pass
            return MockResponse()

        log = RequestMetrics(
            request_id="test-req-1",
            method="GET",
            endpoint="/test"
        )

        # 用于验证指标回调的参数
        metrics_callback_called = False
        assertion_result_received = None
        status_code_received = None
        error_received = None

        async def mock_metrics_callback(**kwargs):
            nonlocal metrics_callback_called, assertion_result_received, status_code_received, error_received
            metrics_callback_called = True
            assertion_result_received = kwargs.get('assertion_result')
            status_code_received = kwargs.get('status_code')
            error_received = kwargs.get('error')

        def mock_log_request(*args, **kwargs):
            pass

        rcm = ResponseContextManager(
            request_coroutine=mock_request(),
            log=log,
            start_time=0,
            metrics_callback=mock_metrics_callback,
            log_request=mock_log_request
        )

        # 测试断言成功但HTTP错误
        async with rcm as response:
            # 断言成功，即使状态码是500
            assert True

        # 验证 assertion_result 被设置为 "pass"
        assert rcm.log.assertion_result == "pass"
        # 验证原始状态码被保留
        assert rcm.log.status_code == 500
        # 验证 metrics_callback 被调用
        assert metrics_callback_called
        # 验证 assertion_result 被正确传递
        assert assertion_result_received == "pass"
        # 验证状态码被正确传递
        assert status_code_received == 500
        # 验证错误信息为None（因为断言成功）
        assert error_received is None

    @allure.story("错误处理")
    @allure.title("测试各种HTTP状态码的指标记录")
    @allure.severity(allure.severity_level.NORMAL)
    @pytest.mark.parametrize("status_code, should_fail", [
        (200, False),
        (201, False),
        (204, False),
        (400, True),
        (401, True),
        (403, True),
        (404, True),
        (500, True),
        (502, True),
        (503, True),
    ])
    async def test_various_status_codes_metrics(
            self, status_code, should_fail):
        """测试各种HTTP状态码的指标记录"""
        # 模拟一个返回指定状态码的响应
        async def mock_request():
            class MockResponse:
                status = status_code
                cookies = {}

                async def read(self):
                    return b'{"message": "test"}'

                def close(self):
                    pass
            return MockResponse()

        log = RequestMetrics(
            request_id="test-req-1",
            method="GET",
            endpoint="/test"
        )

        # 用于验证指标回调的参数
        metrics_callback_called = False
        assertion_result_received = None
        status_code_received = None

        async def mock_metrics_callback(**kwargs):
            nonlocal metrics_callback_called, assertion_result_received, status_code_received
            metrics_callback_called = True
            assertion_result_received = kwargs.get('assertion_result')
            status_code_received = kwargs.get('status_code')

        def mock_log_request(*args, **kwargs):
            pass

        rcm = ResponseContextManager(
            request_coroutine=mock_request(),
            log=log,
            start_time=0,
            metrics_callback=mock_metrics_callback,
            log_request=mock_log_request
        )

        # 测试各种状态码
        if should_fail:
            try:
                async with rcm as response:
                    # 断言失败
                    assert False, f"HTTP {status_code} error"
            except AssertionError:
                pass

            # 验证 assertion_result 被设置为 "fail"
            assert rcm.log.assertion_result == "fail"
        else:
            async with rcm as response:
                # 断言成功
                assert True

            # 验证 assertion_result 被设置为 "pass"
            assert rcm.log.assertion_result == "pass"

        # 验证原始状态码被保留
        assert rcm.log.status_code == status_code
        # 验证 metrics_callback 被调用
        assert metrics_callback_called
        # 验证 assertion_result 被正确传递
        assert assertion_result_received == "fail" if should_fail else assertion_result_received == "pass"
        # 验证状态码被正确传递
        assert status_code_received == status_code


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



# encoding: utf-8

import asyncio

import allure
import pytest

from aiotest.task_manager import TaskManager


@allure.feature("TaskManager")
class TestTaskManager:
    """TaskManager 类的测试用例"""

    @allure.story("初始化")
    @allure.title("测试 TaskManager 初始化")
    @allure.severity(allure.severity_level.NORMAL)
    def test_initialization(self):
        """测试 TaskManager 初始化时正确设置属性"""
        manager = TaskManager()
        assert manager.background_tasks == []

    @allure.story("任务管理")
    @allure.title("测试添加单个任务")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_add_task(self):
        """测试添加单个后台任务"""
        manager = TaskManager()

        async def test_coro():
            await asyncio.sleep(0.1)
            return "test"

        task = asyncio.create_task(test_coro())
        manager.add_task(task)

        assert len(manager.background_tasks) == 1
        assert task in manager.background_tasks

    @allure.story("任务管理")
    @allure.title("测试批量添加任务")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_add_tasks(self):
        """测试批量添加后台任务"""
        manager = TaskManager()

        async def test_coro():
            await asyncio.sleep(0.1)
            return "test"

        tasks = [asyncio.create_task(test_coro()) for _ in range(3)]
        manager.add_tasks(tasks)

        assert len(manager.background_tasks) == 3
        for task in tasks:
            assert task in manager.background_tasks

    @allure.story("任务管理")
    @allure.title("测试创建任务")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_create_task(self):
        """测试创建并添加后台任务"""
        manager = TaskManager()

        async def test_coro():
            await asyncio.sleep(0.1)
            return "test"

        task = manager.create_task(test_coro(), name="test_task")

        assert len(manager.background_tasks) == 1
        assert task in manager.background_tasks
        assert task.get_name() == "test_task"

    @allure.story("任务管理")
    @allure.title("测试取消所有任务")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_cancel_all_tasks(self):
        """测试取消所有后台任务"""
        manager = TaskManager()

        async def test_coro():
            await asyncio.sleep(0.5)
            return "test"

        # 添加多个任务
        for _ in range(3):
            manager.create_task(test_coro())

        assert len(manager.background_tasks) == 3

        # 取消所有任务
        await manager.cancel_all_tasks()

        # 检查任务是否被清理
        for task in manager.background_tasks:
            assert task.done()

    @allure.story("任务管理")
    @allure.title("测试根据名称获取任务")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_get_task_by_name(self):
        """测试根据名称获取任务"""
        manager = TaskManager()

        async def test_coro():
            await asyncio.sleep(0.1)
            return "test"

        # 创建带名称的任务
        task1 = manager.create_task(test_coro(), name="task1")
        task2 = manager.create_task(test_coro(), name="task2")

        # 测试获取存在的任务
        found_task = manager.get_task_by_name("task1")
        assert found_task == task1

        # 测试获取不存在的任务
        not_found_task = manager.get_task_by_name("task3")
        assert not_found_task is None

    @allure.story("任务管理")
    @allure.title("测试移除任务")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_remove_task(self):
        """测试移除任务"""
        manager = TaskManager()

        async def test_coro():
            await asyncio.sleep(0.1)
            return "test"

        task = manager.create_task(test_coro())
        assert len(manager.background_tasks) == 1

        # 移除任务
        manager.remove_task(task)
        assert len(manager.background_tasks) == 0
        assert task not in manager.background_tasks

    @allure.story("任务管理")
    @allure.title("测试清理已完成的任务")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_clear_completed_tasks(self):
        """测试清理已完成的任务"""
        manager = TaskManager()

        async def test_coro():
            await asyncio.sleep(0.1)
            return "test"

        # 创建任务并等待完成
        task1 = manager.create_task(test_coro())
        await task1

        # 创建未完成的任务
        task2 = manager.create_task(asyncio.sleep(0.5))

        assert len(manager.background_tasks) == 2

        # 清理已完成的任务
        manager.clear_completed_tasks()
        assert len(manager.background_tasks) == 1
        assert task2 in manager.background_tasks

    @allure.story("任务管理")
    @allure.title("测试获取活跃任务数量")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_get_active_task_count(self):
        """测试获取活跃任务数量"""
        manager = TaskManager()

        async def test_coro():
            await asyncio.sleep(0.1)
            return "test"

        # 初始状态
        assert manager.get_active_task_count() == 0

        # 添加任务后
        for _ in range(3):
            manager.create_task(test_coro())
        assert manager.get_active_task_count() == 3

        # 等待任务完成后
        await asyncio.sleep(0.2)
        manager.clear_completed_tasks()
        assert manager.get_active_task_count() == 0

    @allure.story("任务管理")
    @allure.title("测试等待任务完成")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_wait_for_task_completion(self):
        """测试等待所有任务完成"""
        manager = TaskManager()

        async def test_coro():
            await asyncio.sleep(0.2)
            return "test"

        # 添加任务
        for _ in range(2):
            manager.create_task(test_coro())

        # 等待任务完成
        start_time = asyncio.get_event_loop().time()
        await manager.wait_for_task_completion()
        end_time = asyncio.get_event_loop().time()

        # 确保任务已完成
        for task in manager.background_tasks:
            assert task.done()
        # 确保等待时间合理（允许一定误差）
        assert end_time - start_time >= 0.15

    @allure.story("任务管理")
    @allure.title("测试等待任务完成超时")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_wait_for_task_completion_timeout(self):
        """测试等待任务完成超时"""
        manager = TaskManager()

        async def test_coro():
            await asyncio.sleep(0.5)
            return "test"

        # 添加任务
        manager.create_task(test_coro())

        # 等待任务完成（超时）
        start_time = asyncio.get_event_loop().time()
        await manager.wait_for_task_completion(timeout=0.1)
        end_time = asyncio.get_event_loop().time()

        # 确保等待时间接近超时时间（允许一定误差）
        assert 0.08 <= end_time - start_time <= 0.25

    @allure.story("任务管理")
    @allure.title("测试根据名称停止任务")
    @allure.severity(allure.severity_level.NORMAL)
    async def test_stop_task_by_name(self):
        """测试根据名称停止任务"""
        manager = TaskManager()

        async def test_coro():
            await asyncio.sleep(0.5)
            return "test"

        # 创建带名称的任务
        manager.create_task(test_coro(), name="test_task")

        # 停止任务
        result = await manager.stop_task_by_name("test_task")
        assert result is True

        # 测试停止不存在的任务
        result = await manager.stop_task_by_name("non_existent")
        assert result is False


if __name__ == "__main__":
    pytest.main([__file__, "-v"])
